#
# Copyright (C) 2020 Pico Technology Ltd. See LICENSE file for terms.
#
# PS6000 A BLOCK MODE EXAMPLE
# This example opens a 6000a driver device, sets up two channels and a trigger then collects a block of data.
# This data is then plotted as mV against time in ns.

import ctypes
import numpy as np
from picosdk.ps6000a import ps6000a as ps
from picosdk.PicoDeviceEnums import picoEnum as enums
from picosdk.PicoDeviceStructs import picoStruct as struct
import matplotlib.pyplot as plt
from picosdk.functions import adc2mV, assert_pico_ok, mV2adc

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# Open 6000 A series PicoScope
# returns handle to chandle for use in future API functions
resolution = enums.PICO_DEVICE_RESOLUTION["PICO_DR_8BIT"]
status["openunit"] = ps.ps6000aOpenUnit(ctypes.byref(chandle), None, resolution)
assert_pico_ok(status["openunit"])

# Set channel A on
# handle = chandle
channelA = enums.PICO_CHANNEL["PICO_CHANNEL_A"]
coupling = enums.PICO_COUPLING["PICO_DC"]
channelRange = 7
# analogueOffset = 0 V
bandwidth = enums.PICO_BANDWIDTH_LIMITER["PICO_BW_FULL"]
status["setChannelA"] = ps.ps6000aSetChannelOn(chandle, channelA, coupling, channelRange, 0, bandwidth)
assert_pico_ok(status["setChannelA"])

channelB = enums.PICO_CHANNEL["PICO_CHANNEL_B"]
status["setChannelB"] = ps.ps6000aSetChannelOn(chandle, channelB, coupling, channelRange, 0, bandwidth)
assert_pico_ok(status["setChannelB"])

# set channel C-H off
for x in range (2, 7, 1):
    channel = x
    status["setChannel",x] = ps.ps6000aSetChannelOff(chandle,channel)
    assert_pico_ok(status["setChannel",x])
    
# get max ADC value
# handle = chandle
minADC = ctypes.c_int16()
maxADC = ctypes.c_int16()
status["getAdcLimits"] = ps.ps6000aGetAdcLimits(chandle, resolution, ctypes.byref(minADC), ctypes.byref(maxADC))
assert_pico_ok(status["getAdcLimits"])

# use the trigger functions seperately
# set up a simple edge trigger on channel A OR B with a 1 V threshold

conditions = (struct.PICO_CONDITION * 2)()
conditions[0] = struct.PICO_CONDITION(channelA,enums.PICO_TRIGGER_STATE["PICO_CONDITION_TRUE"])
conditions[1] = struct.PICO_CONDITION(channelB, enums.PICO_TRIGGER_STATE["PICO_CONDITION_TRUE"])
nConditions = 2
clear = enums.PICO_ACTION["PICO_CLEAR_ALL"]
add = enums.PICO_ACTION["PICO_ADD"]
action = clear|add # PICO_ACTION["PICO_CLEAR_WAVEFORM_CLEAR_ALL"] | PICO_ACTION["PICO_ADD"]  
status["setTriggerChannelConditions"] = ps.ps6000aSetTriggerChannelConditions(chandle, ctypes.byref(conditions), nConditions, action)
assert_pico_ok(status["setTriggerChannelConditions"])

directions = (struct.PICO_DIRECTION * 2)()
directions[0]= struct.PICO_DIRECTION(channelA, enums.PICO_THRESHOLD_DIRECTION["PICO_RISING"], enums.PICO_THRESHOLD_MODE["PICO_LEVEL"])
directions[1]= struct.PICO_DIRECTION(channelB, enums.PICO_THRESHOLD_DIRECTION["PICO_RISING"], enums.PICO_THRESHOLD_MODE["PICO_LEVEL"])
nDirections = 2
status["setTriggerChannelDirections"] = ps.ps6000aSetTriggerChannelDirections(chandle,ctypes.byref(directions),nDirections)
assert_pico_ok(status["setTriggerChannelDirections"])

channelProperties = (struct.PICO_TRIGGER_CHANNEL_PROPERTIES * 2)()
channelProperties[0] = struct.PICO_TRIGGER_CHANNEL_PROPERTIES(mV2adc(1000,channelRange,maxADC), 0, 0, 0, channelA)
channelProperties[1] = struct.PICO_TRIGGER_CHANNEL_PROPERTIES(mV2adc(1000,channelRange,maxADC), 0, 0, 0, channelB)
nChannelProperties = 2
autoTriggerMicroSeconds = 1000000
status["setTriggerChannelProperties"] = ps.ps6000aSetTriggerChannelProperties(chandle, ctypes.byref(channelProperties),nChannelProperties,0,autoTriggerMicroSeconds)
assert_pico_ok(status["setTriggerChannelProperties"])

# Get fastest available timebase
# handle = chandle
enabledChannelFlags = enums.PICO_CHANNEL_FLAGS["PICO_CHANNEL_A_FLAGS"] + enums.PICO_CHANNEL_FLAGS["PICO_CHANNEL_B_FLAGS"]
timebase = ctypes.c_uint32(0)
timeInterval = ctypes.c_double(0)
# resolution = resolution
status["getMinimumTimebaseStateless"] = ps.ps6000aGetMinimumTimebaseStateless(chandle, enabledChannelFlags, ctypes.byref(timebase), ctypes.byref(timeInterval), resolution)
print("timebase = ", timebase.value)
print("sample interval =", timeInterval.value, "s")

# Set number of samples to be collected
noOfPreTriggerSamples = 500000
noOfPostTriggerSamples = 1000000
nSamples = noOfPostTriggerSamples + noOfPreTriggerSamples

# Create buffers
bufferAMax = (ctypes.c_int16 * nSamples)()
bufferAMin = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example
bufferBMax = (ctypes.c_int16 * nSamples)()
bufferBMin = (ctypes.c_int16 * nSamples)() # used for downsampling which isn't in the scope of this example

# Set data buffers
# handle = chandle
# channel = channelA
# bufferMax = bufferAMax
# bufferMin = bufferAMin
# nSamples = nSamples
dataType = enums.PICO_DATA_TYPE["PICO_INT16_T"]
waveform = 0
downSampleMode = enums.PICO_RATIO_MODE["PICO_RATIO_MODE_RAW"]
action = clear|add # PICO_ACTION["PICO_CLEAR_WAVEFORM_CLEAR_ALL"] | PICO_ACTION["PICO_ADD"]  
status["setDataBuffers"] = ps.ps6000aSetDataBuffers(chandle, channelA, ctypes.byref(bufferAMax), ctypes.byref(bufferAMin), nSamples, dataType, waveform, downSampleMode, action)
assert_pico_ok(status["setDataBuffers"])
status["setDataBuffers"] = ps.ps6000aSetDataBuffers(chandle, channelB, ctypes.byref(bufferBMax), ctypes.byref(bufferBMin), nSamples, dataType, waveform, downSampleMode, action)
assert_pico_ok(status["setDataBuffers"])

# Run block capture
# handle = chandle
# timebase = timebase
timeIndisposedMs = ctypes.c_double(0)
# segmentIndex = 0
# lpReady = None   Using IsReady rather than a callback
# pParameter = None
status["runBlock"] = ps.ps6000aRunBlock(chandle, noOfPreTriggerSamples, noOfPostTriggerSamples, timebase, ctypes.byref(timeIndisposedMs), 0, None, None)
assert_pico_ok(status["runBlock"])

# Check for data collection to finish using ps6000aIsReady
ready = ctypes.c_int16(0)
check = ctypes.c_int16(0)
while ready.value == check.value:
    status["isReady"] = ps.ps6000aIsReady(chandle, ctypes.byref(ready))
    
# Get data from scope
# handle = chandle
# startIndex = 0
noOfSamples = ctypes.c_uint64(nSamples)
# downSampleRatio = 1
# segmentIndex = 0
overflow = ctypes.c_int16(0)
status["getValues"] = ps.ps6000aGetValues(chandle, 0, ctypes.byref(noOfSamples), 1, downSampleMode, 0, ctypes.byref(overflow))
assert_pico_ok(status["getValues"])

# convert ADC counts data to mV
adc2mVChAMax =  adc2mV(bufferAMax, channelRange, maxADC)
adc2mVChBMax =  adc2mV(bufferBMax, channelRange, maxADC)

# Create time data
time = np.linspace(0, (nSamples -1) * timeInterval.value * 1000000000, nSamples)

# plot data from channel A and B
plt.plot(time, adc2mVChAMax[:])
plt.plot(time, adc2mVChBMax[:])
plt.xlabel('Time (ns)')
plt.ylabel('Voltage (mV)')
plt.show()

# Close the scope
status["closeunit"] = ps.ps6000aCloseUnit(chandle)
assert_pico_ok(status["closeunit"])

print(status)


#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
Unit tests for the wrapper function for retrieving device information.
"""

from __future__ import print_function

from test.test_helpers import DriverTest, drivers_with_device_connected
from picosdk.errors import DeviceNotFoundError, ArgumentOutOfRangeError

try:
    STRING_TYPES = (str, unicode)
except:
    STRING_TYPES = (bytes, str)


class GetInfoTest(DriverTest):
    def test_get_info_success(self):
        """test_get_info_success
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return
        drivers_to_use = drivers_with_device_connected

        def test(driver):
            devices = []
            try:
                devices.append(driver.open_unit())
            except DeviceNotFoundError:
                return "no device found."

            device = devices.pop()
            info = driver.get_unit_info(device)
            device.close()

            self.assertEqual(info.driver, driver)
            self.assertIsInstance(info.variant, STRING_TYPES)
            self.assertIsInstance(info.serial, STRING_TYPES)

        self.run_snippet_and_count_problems(drivers_to_use, test)

    def test_get_info_advanced_success(self):
        """test_get_info_advanced_success
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return
        drivers_to_use = drivers_with_device_connected

        def test(driver):
            devices = []
            try:
                devices.append(driver.open_unit())
            except DeviceNotFoundError:
                return "no device found."

            device = devices.pop()
            # three keys present on ALL devices.
            info = driver.get_unit_info(device,
                                        "PICO_DRIVER_VERSION",
                                        "PICO_USB_VERSION",
                                        "PICO_HARDWARE_VERSION")
            device.close()

            self.assertEqual(len(info), 3)
            self.assertIsInstance(info.PICO_DRIVER_VERSION, STRING_TYPES)
            self.assertIsInstance(info.PICO_USB_VERSION, STRING_TYPES)
            self.assertIsInstance(info.PICO_HARDWARE_VERSION, STRING_TYPES)

        self.run_snippet_and_count_problems(drivers_to_use, test)

    def test_get_info_with_invalid_key_fails(self):
        """test_get_info_advanced_success
        note: test assumes you have set test_helpers.drivers_with_device_connected"""
        if not drivers_with_device_connected:
            return
        drivers_to_use = drivers_with_device_connected

        def test(driver):
            devices = []
            try:
                devices.append(driver.open_unit())
            except DeviceNotFoundError:
                return "no device found."

            device = devices.pop()
            threw = False
            try:
                # some valid and some invalid keys. People may pass PICO_CAL_DATE to ps2000 etc.
                info = driver.get_unit_info(device,
                                            "PICO_DRIVER_VERSION",
                                            "aillk_jasgKLASDFG_Jlasdkgfj",
                                            "PICO_HARDWARE_VERSION")
            except ArgumentOutOfRangeError:
                threw = True
            finally:
                device.close()

            if not threw:
                return "get_unit_info didn't throw exception on bad key."

        self.run_snippet_and_count_problems(drivers_to_use, test)



#
# Copyright (C) 2014-2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the ps5000aApi.h C header
file for PicoScope 5000 Series oscilloscopes using the ps5000a driver API
functions.
"""

from ctypes import *
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.library import Library
from picosdk.constants import make_enum


class Ps5000alib(Library):
    def __init__(self):
        super(Ps5000alib, self).__init__("ps5000a")


ps5000a = Ps5000alib()

ps5000a.PS5000A_DEVICE_RESOLUTION = make_enum([
    "PS5000A_DR_8BIT",
    "PS5000A_DR_12BIT",
    "PS5000A_DR_14BIT",
    "PS5000A_DR_15BIT",
    "PS5000A_DR_16BIT",
])

ps5000a.DEFAULT_RESOLUTION = ps5000a.PS5000A_DEVICE_RESOLUTION["PS5000A_DR_8BIT"]

ps5000a.PS5000A_COUPLING = make_enum([
    'PS5000A_AC',
    'PS5000A_DC',
])

# Just use AC and DC.
ps5000a.PICO_COUPLING = {k[-2:]: v for k, v in ps5000a.PS5000A_COUPLING.items()}

def _define_channel():
    PS5000A_CHANNEL_A = 0
    PS5000A_CHANNEL_B = 1
    PS5000A_CHANNEL_C = 2
    PS5000A_CHANNEL_D = 3
    PS5000A_EXTERNAL = PS5000A_MAX_CHANNELS = 4
    PS5000A_TRIGGER_AUX = 5
    PS5000A_MAX_TRIGGER_SOURCE = 6
    PS5000A_DIGITAL_PORT0 = 0x80
    PS5000A_DIGITAL_PORT1 = PS5000A_DIGITAL_PORT0 + 1
    PS5000A_DIGITAL_PORT2 = PS5000A_DIGITAL_PORT0 + 2
    PS5000A_DIGITAL_PORT3 = PS5000A_DIGITAL_PORT0 + 3
    PS5000A_PULSE_WIDTH_SOURCE = 0x10000000
	
    return {k.upper(): v for k, v in locals().items() if k.startswith("PS5000A")}

ps5000a.PS5000A_CHANNEL = _define_channel()

# only include the normal analog channels for now:
ps5000a.PICO_CHANNEL = {k[-1]: v for k, v in ps5000a.PS5000A_CHANNEL.items() if "PS5000A_CHANNEL_" in k}

ps5000a.PS5000A_RANGE = make_enum([
    "PS5000A_10MV",
    "PS5000A_20MV",
    "PS5000A_50MV",
    "PS5000A_100MV",
    "PS5000A_200MV",
    "PS5000A_500MV",
    "PS5000A_1V",
    "PS5000A_2V",
    "PS5000A_5V",
    "PS5000A_10V",
    "PS5000A_20V",
    "PS5000A_50V",
    "PS5000A_MAX_RANGES",
])


ps5000a.PS5000A_THRESHOLD_DIRECTION = make_enum([
    ("PS5000A_ABOVE", "PS5000A_INSIDE"),
    ("PS5000A_BELOW", "PS5000A_OUTSIDE"),
    ("PS5000A_RISING", "PS5000A_ENTER", "PS5000A_NONE"),
    ("PS5000A_FALLING", "PS5000A_EXIT"),
    ("PS5000A_RISING_OR_FALLING", "PS5000A_ENTER_OR_EXIT"),
    "PS5000A_ABOVE_LOWER",
    "PS5000A_BELOW_LOWER",
    "PS5000A_RISING_LOWER",
    "PS5000A_FALLING_LOWER",
    "PS5000A_POSITIVE_RUNT",
    "PS5000A_NEGATIVE_RUNT",
])


ps5000a.PICO_VOLTAGE_RANGE = {
    v: float(k.split('_')[1][:-1]) if k[-2] != 'M' else (0.001 * float(k.split('_')[1][:-2]))
    for k, v in ps5000a.PS5000A_RANGE.items() if k != "PS5000A_MAX_RANGES"
}

ps5000a.PS5000A_DIGITAL_CHANNEL = make_enum([
    "PS5000A_DIGITAL_CHANNEL_0",
    "PS5000A_DIGITAL_CHANNEL_1",
    "PS5000A_DIGITAL_CHANNEL_2",
    "PS5000A_DIGITAL_CHANNEL_3",
    "PS5000A_DIGITAL_CHANNEL_4",
    "PS5000A_DIGITAL_CHANNEL_5",
    "PS5000A_DIGITAL_CHANNEL_6",
    "PS5000A_DIGITAL_CHANNEL_7",
    "PS5000A_DIGITAL_CHANNEL_8",
    "PS5000A_DIGITAL_CHANNEL_9",
    "PS5000A_DIGITAL_CHANNEL_10",
    "PS5000A_DIGITAL_CHANNEL_11",
    "PS5000A_DIGITAL_CHANNEL_12",
    "PS5000A_DIGITAL_CHANNEL_13",
    "PS5000A_DIGITAL_CHANNEL_14",
    "PS5000A_DIGITAL_CHANNEL_15",
    "PS5000A_DIGITAL_CHANNEL_16",
    "PS5000A_DIGITAL_CHANNEL_17",
    "PS5000A_DIGITAL_CHANNEL_18",
    "PS5000A_DIGITAL_CHANNEL_19",
    "PS5000A_DIGITAL_CHANNEL_20",
    "PS5000A_DIGITAL_CHANNEL_21",
    "PS5000A_DIGITAL_CHANNEL_22",
    "PS5000A_DIGITAL_CHANNEL_23",
    "PS5000A_DIGITAL_CHANNEL_24",
    "PS5000A_DIGITAL_CHANNEL_25",
    "PS5000A_DIGITAL_CHANNEL_26",
    "PS5000A_DIGITAL_CHANNEL_27",
    "PS5000A_DIGITAL_CHANNEL_28",
    "PS5000A_DIGITAL_CHANNEL_29",
    "PS5000A_DIGITAL_CHANNEL_30",
    "PS5000A_DIGITAL_CHANNEL_31",
    "PS5000A_MAX_DIGITAL_CHANNELS"
])

ps5000a.PS5000A_DIGITAL_DIRECTION = make_enum([
    "PS5000A_DIGITAL_DONT_CARE",
    "PS5000A_DIGITAL_DIRECTION_LOW",
    "PS5000A_DIGITAL_DIRECTION_HIGH",
    "PS5000A_DIGITAL_DIRECTION_RISING",
    "PS5000A_DIGITAL_DIRECTION_FALLING",
    "PS5000A_DIGITAL_DIRECTION_RISING_OR_FALLING",
    "PS5000A_DIGITAL_MAX_DIRECTION"
])

def _define_conditionsInfo():
    PS5000A_CLEAR = 0x00000001
    PS5000A_ADD = 0x00000002

    return {k.upper(): v for k, v in locals().items() if k.startswith("PS5000A")}

PS5000AConditionsInfo = _define_conditionsInfo()

ps5000a.PS5000A_THRESHOLD_MODE = make_enum([
    "PS5000A_LEVEL",
    "PS5000A_WINDOW"
])

ps5000a.PS5000A_TRIGGER_STATE = make_enum([
	"PS5000A_CONDITION_DONT_CARE",
	"PS5000A_CONDITION_TRUE",
	"PS5000A_CONDITION_FALSE",
	"PS5000A_CONDITION_MAX"
])

ps5000a.PS5000A_RATIO_MODE = {
    'PS5000A_RATIO_MODE_NONE': 0,
    'PS5000A_RATIO_MODE_AGGREGATE': 1,
    'PS5000A_RATIO_MODE_DECIMATE': 2,
    'PS5000A_RATIO_MODE_AVERAGE': 4,
}

ps5000a.PS5000A_TIME_UNITS = make_enum([
    'PS5000A_FS',
    'PS5000A_PS',
    'PS5000A_NS',
    'PS5000A_US',
    'PS5000A_MS',
    'PS5000A_S',
    'PS5000A_MAX_TIME_UNITS',
])

ps5000a.PICO_RATIO_MODE = {k[19:]: v for k, v in ps5000a.PS5000A_RATIO_MODE.items()}



class PS5000A_TRIGGER_INFO(Structure):
    _pack_ = 1
    _fields_ = [("status", c_uint32),
                ("segmentIndex", c_uint32),
                ("triggerIndex", c_uint32),
                ("triggerTime", c_int64),
                ("timeUnits", c_int16),
                ("reserved0", c_int16),
                ("timeStampCounter", c_uint64)]

ps5000a.PS5000A_TRIGGER_INFO = PS5000A_TRIGGER_INFO

class PS5000A_DIGITAL_CHANNEL_DIRECTIONS(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_int32),
                ("direction", c_int32)]

ps5000a.PS5000A_DIGITAL_CHANNEL_DIRECTIONS = PS5000A_DIGITAL_CHANNEL_DIRECTIONS
				
class PS5000A_DIRECTION(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_int32),
                ("direction", c_int32),
                ("mode", c_int32)]
                
ps5000a.PS5000A_DIRECTION = PS5000A_DIRECTION

				
class PS5000A_TRIGGER_CHANNEL_PROPERTIES_V2(Structure):
    _pack_ = 1
    _fields_ = [("thresholdUpper", c_int16),
                ("thresholdUpperHysteresis", c_uint16),
                ("thresholdLower", c_int16),
                ("thresholdLowerHysteresis", c_uint16),
                ("channel", c_int32)]
                
ps5000a.PS5000A_TRIGGER_CHANNEL_PROPERTIES_V2 = PS5000A_TRIGGER_CHANNEL_PROPERTIES_V2
				
				
class PS5000A_CONDITION (Structure):
	_pack_ = 1
	_fields_ = [("source", c_int32),
				("condition", c_int16)]
                
ps5000a.PS5000A_CONDITION = PS5000A_CONDITION

class PS5000A_PWQ_CONDITIONS (Structure):
    _pack_ = 1
    _fields_ = [("channelA", c_int16),
                ("channelB", c_int16),
                ("channelC", c_int16),
                ("channelD", c_int16),
                ("external", c_int16),
                ("aux", c_int16)]

ps5000a.PS5000A_PWQ_CONDITIONS = PS5000A_PWQ_CONDITIONS

doc = """ PICO_STATUS (ps5000aOpenUnit)
    (
        int16_t                   *handle,
        int8_t                    *serial,
        PS5000A_DEVICE_RESOLUTION  resolution
    ); """
ps5000a.make_symbol("_OpenUnit", "ps5000aOpenUnit", c_uint32, [c_void_p, c_char_p, c_int32], doc)

doc = """ PICO_STATUS ps5000aOpenUnitAsync
    (
        int16_t                   *status,
        int8_t                    *serial,
        PS5000A_DEVICE_RESOLUTION  resolution
    ); """
ps5000a.make_symbol("_OpenUnitAsync", "ps5000aOpenUnitAsync", c_uint32, [c_void_p, c_char_p, c_int32], doc)

doc = """ PICO_STATUS ps5000aOpenUnitProgress
    (
        int16_t *handle,
        int16_t *progressPercent,
        int16_t *complete
    ); """
ps5000a.make_symbol("_OpenUnitProgress", "ps5000aOpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetUnitInfo
    (
        int16_t   handle,
        int8_t   *string,
        int16_t   stringLength,
        int16_t  *requiredSize,
        PICO_INFO info
    ); """
ps5000a.make_symbol("_GetUnitInfo", "ps5000aGetUnitInfo", c_uint32, [c_int16, c_char_p, c_int16, c_void_p, c_uint32],
                    doc)

doc = """ PICO_STATUS ps5000aFlashLed
    (
        int16_t handle,
        int16_t start
    ); """
ps5000a.make_symbol("_FlashLed", "ps5000aFlashLed", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps5000aIsLedFlashing
    (
        int16_t  handle,
        int16_t *status
    ); """
ps5000a.make_symbol("_IsLedFlashing", "ps5000aIsLedFlashing", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aCloseUnit
    (
        int16_t handle
    ); """
ps5000a.make_symbol("_CloseUnit", "ps5000aCloseUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps5000aMemorySegments
    (
        int16_t   handle,
        uint32_t  nSegments,
        int32_t  *nMaxSamples
    ); """
ps5000a.make_symbol("_MemorySegments", "ps5000aMemorySegments", c_uint32, [c_int16, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps5000aSetChannel
    (
        int16_t          handle,
        PS5000a_CHANNEL  channel,
        int16_t          enabled,
        PS5000a_COUPLING type,
        PS5000a_RANGE    range,
        float            analogOffset
    ); """
ps5000a.make_symbol("_SetChannel", "ps5000aSetChannel", c_uint32,
                    [c_int16, c_int32, c_int16, c_int32, c_int32, c_float], doc)

doc = """ PICO_STATUS ps5000aSetBandwidthFilter
    (
        int16_t                    handle,
        PS5000A_CHANNEL            channel,
        PS5000A_BANDWIDTH_LIMITER  bandwidth
    ); """
ps5000a.make_symbol("_SetBandwidthFilter", "ps5000aSetBandwidthFilter", c_uint32, [c_int16, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps5000aGetTimebase
    (
        int16_t   handle,
        uint32_t  timebase,
        int32_t   noSamples,
        int32_t  *timeIntervalNanoseconds,
        int32_t  *maxSamples,
        uint32_t  segmentIndex
    ); """
ps5000a.make_symbol("_GetTimebase", "ps5000aGetTimebase", c_uint32,
                    [c_int16, c_uint32, c_int32, c_void_p, c_void_p, c_uint32], doc)


doc = """ PICO_STATUS ps5000aGetTimebase2
    (
        int16_t   handle,
        uint32_t  timebase,
        int32_t   noSamples,
        float    *timeIntervalNanoseconds,
        int32_t  *maxSamples,
        uint32_t  segmentIndex
    ); """
ps5000a.make_symbol("_GetTimebase2", "ps5000aGetTimebase2", c_uint32,
                    [c_int16, c_uint32, c_int32, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps5000aSetSigGenArbitrary
    (
        int16_t                     handle,
        int32_t                     offsetVoltage,
        uint32_t                    pkToPk,
        uint32_t                    startDeltaPhase,
        uint32_t                    stopDeltaPhase,
        uint32_t                    deltaPhaseIncrement,
        uint32_t                    dwellCount,
        int16_t                    *arbitraryWaveform,
        int32_t                     arbitraryWaveformSize,
        PS5000A_SWEEP_TYPE          sweepType,
        PS5000A_EXTRA_OPERATIONS    operation,
        PS5000A_INDEX_MODE          indexMode,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS5000A_SIGGEN_TRIG_TYPE    triggerType,
        PS5000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps5000a.make_symbol("_SetSigGenArbitrary", "ps5000aSetSigGenArbitrary", c_uint32,
                    [c_int16, c_int32, c_uint32, c_uint32, c_uint32, c_uint32, c_uint32, c_void_p, c_int32, c_int32,
                     c_int32, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps5000aSetSigGenBuiltInV2
    (
        int16_t                     handle,
        int32_t                     offsetVoltage,
        uint32_t                    pkToPk,
        PS5000A_WAVE_TYPE           waveType,
        double                      startFrequency,
        double                      stopFrequency,
        double                      increment,
        double                      dwellTime,
        PS5000A_SWEEP_TYPE          sweepType,
        PS5000A_EXTRA_OPERATIONS    operation,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS5000A_SIGGEN_TRIG_TYPE    triggerType,
        PS5000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps5000a.make_symbol("_SetSigGenBuiltIn", "ps5000aSetSigGenBuiltInV2", c_uint32,
                    [c_int16, c_int32, c_uint32, c_int32, c_double, c_double, c_double, c_double,
                     c_int32, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps5000aSetSigGenPropertiesArbitrary
    (
        int16_t                     handle,
        uint32_t                    startDeltaPhase,
        uint32_t                    stopDeltaPhase,
        uint32_t                    deltaPhaseIncrement,
        uint32_t                    dwellCount,
        PS5000A_SWEEP_TYPE          sweepType,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS5000A_SIGGEN_TRIG_TYPE    triggerType,
        PS5000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps5000a.make_symbol("_SetSigGenPropertiesArbitrary", "ps5000aSetSigGenPropertiesArbitrary", c_uint32,
                    [c_int16, c_uint32, c_uint32, c_uint32, c_uint32, c_int32, c_uint32, c_uint32, c_int32, c_int32,
                     c_int16], doc)

doc = """ PICO_STATUS ps5000aSetSigGenPropertiesBuiltIn
    (
        int16_t                     handle,
        double                      startFrequency,
        double                      stopFrequency,
        double                      increment,
        double                      dwellTime,
        PS5000A_SWEEP_TYPE          sweepType,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS5000A_SIGGEN_TRIG_TYPE    triggerType,
        PS5000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps5000a.make_symbol("_SetSigGenPropertiesBuiltIn", "ps5000aSetSigGenPropertiesBuiltIn", c_uint32,
                    [c_int16, c_double, c_double, c_double, c_double, c_int32, c_uint32, c_uint32, c_int32, c_int32,
                     c_int16], doc)

doc = """ PICO_STATUS ps5000aSigGenFrequencyToPhase
    (
        int16_t             handle,
        double              frequency,
        PS5000A_INDEX_MODE  indexMode,
        uint32_t            bufferLength,
        uint32_t           *phase
    ); """
ps5000a.make_symbol("_SigGenFrequencyToPhase", "ps5000aSigGenFrequencyToPhase", c_uint32,
                    [c_int16, c_double, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps5000aSigGenArbitraryMinMaxValues
    (
        int16_t   handle,
        int16_t  *minArbitraryWaveformValue,
        int16_t  *maxArbitraryWaveformValue,
        uint32_t *minArbitraryWaveformSize,
        uint32_t *maxArbitraryWaveformSize
    ); """
ps5000a.make_symbol("_SigGenArbitraryMinMaxValues", "ps5000aSigGenArbitraryMinMaxValues", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000aSigGenSoftwareControl
    (
        int16_t  handle,
        int16_t  state
    ); """
ps5000a.make_symbol("_SigGenSoftwareControl", "ps5000aSigGenSoftwareControl", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps5000aSetEts
    (
        int16_t           handle,
        PS5000A_ETS_MODE  mode,
        int16_t           etsCycles,
        int16_t           etsInterleave,
        int32_t          *sampleTimePicoseconds
    ); """
ps5000a.make_symbol("_SetEts", "ps5000aSetEts", c_uint32, [c_int16, c_int32, c_int16, c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aSetTriggerChannelProperties
    (
        int16_t                             handle,
        PS5000A_TRIGGER_CHANNEL_PROPERTIES *channelProperties,
        int16_t                             nChannelProperties,
        int16_t                             auxOutputEnable,
        int32_t                             autoTriggerMilliseconds
    ); """
ps5000a.make_symbol("_SetTriggerChannelProperties", "ps5000aSetTriggerChannelProperties", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps5000aSetTriggerChannelPropertiesV2
    (
        int16_t                             handle,
        PS5000A_TRIGGER_CHANNEL_PROPERTIESV2 *channelProperties,
        int16_t                             nChannelProperties,
        int16_t                             auxOutputEnable,
    ); """
ps5000a.make_symbol("_SetTriggerChannelPropertiesV2", "ps5000aSetTriggerChannelPropertiesV2", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int16], doc)
                    
doc = """ PICO_STATUS ps5000aSetAutoTriggerMicroSeconds
    (
        int16_t                             handle,
        uint64_t                          autoTriggerMicroseconds,
    ); """
ps5000a.make_symbol("_SetAutoTriggerMicroSeconds", "ps5000aSetAutoTriggerMicroSeconds", c_uint32,
                    [c_int16, c_uint64], doc)

doc = """ PICO_STATUS ps5000aSetTriggerChannelConditions
    (
        int16_t                     handle,
        PS5000A_TRIGGER_CONDITIONS *conditions,
        int16_t                     nConditions
    ); """
ps5000a.make_symbol("_SetTriggerChannelConditions", "ps5000aSetTriggerChannelConditions", c_uint32,
                    [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps5000aSetTriggerChannelConditionsV2
    (
        int16_t                     handle,
        PS5000A_TRIGGER_CONDITIONS *conditions,
        int16_t                     nConditions
        PS5000A_CONDITIONS_INFO     info
    ); """
ps5000a.make_symbol("_SetTriggerChannelConditionsV2", "ps5000aSetTriggerChannelConditionsV2", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps5000aSetTriggerChannelDirections
    (
        int16_t                      handle,
        PS5000A_THRESHOLD_DIRECTION  channelA,
        PS5000A_THRESHOLD_DIRECTION  channelB,
        PS5000A_THRESHOLD_DIRECTION  channelC,
        PS5000A_THRESHOLD_DIRECTION  channelD,
        PS5000A_THRESHOLD_DIRECTION  ext,
        PS5000A_THRESHOLD_DIRECTION  aux
    ); """
ps5000a.make_symbol("_SetTriggerChannelDirections", "ps5000aSetTriggerChannelDirections", c_uint32,
                    [c_int16, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps5000aSetTriggerChannelDirectionsV2
    (
        int16_t                    handle,
        PS5000A_DIRECTION          *directions
        uint16_t                   nDirections
     ); """
ps5000a.make_symbol("_SetTriggerChannelDirectionsV2", "ps5000aSetTriggerChannelDirectionsV2", c_uint32,
                   [c_int16, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps5000aSetSimpleTrigger
    (
        int16_t                      handle,
        int16_t                      enable,
        PS5000A_CHANNEL              source,
        int16_t                      threshold,
        PS5000A_THRESHOLD_DIRECTION  direction,
        uint32_t                     delay,
        int16_t                      autoTrigger_ms
    ); """
ps5000a.make_symbol("_SetSimpleTrigger", "ps5000aSetSimpleTrigger", c_uint32,
                    [c_int16, c_int16, c_int32, c_int16, c_int32, c_uint32, c_int16], doc)

doc = """ PICO_STATUS ps5000aSetTriggerDelay
    (
        int16_t   handle,
        uint32_t  delay
    ); """
ps5000a.make_symbol("_SetTriggerDelay", "ps5000aSetTriggerDelay", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps5000aSetPulseWidthQualifier
    (
        int16_t                      handle,
        PS5000A_PWQ_CONDITIONS      *conditions,
        int16_t                      nConditions,
        PS5000A_THRESHOLD_DIRECTION  direction,
        uint32_t                     lower,
        uint32_t                     upper,
        PS5000A_PULSE_WIDTH_TYPE     type
    ); """
ps5000a.make_symbol("_SetPulseWidthQualifier", "ps5000aSetPulseWidthQualifier", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int32, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps5000aIsTriggerOrPulseWidthQualifierEnabled
    (
        int16_t  handle,
        int16_t *triggerEnabled,
        int16_t *pulseWidthQualifierEnabled
    ); """
ps5000a.make_symbol("_IsTriggerOrPulseWidthQualifierEnabled", "ps5000aIsTriggerOrPulseWidthQualifierEnabled", c_uint32,
                    [c_int16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetTriggerTimeOffset64
    (
        int16_t             handle,
        int64_t            *time,
        PS5000A_TIME_UNITS *timeUnits,
        uint32_t            segmentIndex
    ); """
ps5000a.make_symbol("_GetTriggerTimeOffset", "ps5000aGetTriggerTimeOffset64", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps5000aGetValuesTriggerTimeOffsetBulk64
    (
        int16_t             handle,
        int64_t            *times,
        PS5000A_TIME_UNITS *timeUnits,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex
    ); """
ps5000a.make_symbol("_GetValuesTriggerTimeOffsetBulk", "ps5000aGetValuesTriggerTimeOffsetBulk64", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps5000aSetDataBuffers
    (
        int16_t            handle,
        PS5000A_CHANNEL    channel,
        int16_t           *bufferMax,
        int16_t           *bufferMin,
        int32_t            bufferLth,
        uint32_t           segmentIndex,
        PS5000A_RATIO_MODE mode
    ); """
ps5000a.make_symbol("_SetDataBuffers", "ps5000aSetDataBuffers", c_uint32,
                    [c_int16, c_int32, c_void_p, c_void_p, c_int32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps5000aSetDataBuffer
    (
        int16_t            handle,
        PS5000A_CHANNEL    channel,
        int16_t           *buffer,
        int32_t            bufferLth,
        uint32_t           segmentIndex,
        PS5000A_RATIO_MODE mode
    ); """
ps5000a.make_symbol("_SetDataBuffer", "ps5000aSetDataBuffer", c_uint32,
                    [c_int16, c_int32, c_void_p, c_int32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps5000aSetEtsTimeBuffer
    (
        int16_t  handle,
        int64_t *buffer,
        int32_t  bufferLth
    ); """
ps5000a.make_symbol("_SetEtsTimeBuffer", "ps5000aSetEtsTimeBuffer", c_uint32, [c_int16, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps5000aIsReady
    (
        int16_t  handle,
        int16_t *ready
    ); """
ps5000a.make_symbol("_IsReady", "ps5000aIsReady", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aRunBlock
    (
        int16_t            handle,
        int32_t            noOfPreTriggerSamples,
        int32_t            noOfPostTriggerSamples,
        uint32_t           timebase,
        int32_t           *timeIndisposedMs,
        uint32_t           segmentIndex,
        ps5000aBlockReady  lpReady,
        void              *pParameter
    ); """
ps5000a.make_symbol("_RunBlock", "ps5000aRunBlock", c_uint32,
                    [c_int16, c_int32, c_int32, c_uint32, c_void_p, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000aRunStreaming
    (
        int16_t             handle,
        uint32_t            *sampleInterval,
        PS5000A_TIME_UNITS  sampleIntervalTimeUnits,
        uint32_t            maxPreTriggerSamples,
        uint32_t            maxPostPreTriggerSamples,
        int16_t             autoStop,
        uint32_t            downSampleRatio,
        PS5000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            overviewBufferSize
    ); """
ps5000a.make_symbol("_RunStreaming", "ps5000aRunStreaming", c_uint32,
                    [c_int16, c_void_p, c_int32, c_uint32, c_uint32, c_int16, c_uint32, c_int32, c_uint32], doc)

doc = """ PICO_STATUS ps5000aGetStreamingLatestValues
    (
        int16_t                handle,
        ps5000aStreamingReady  lpPs5000aReady,
        void                  *pParameter
    ); """
ps5000a.make_symbol("_GetStreamingLatestValues", "ps5000aGetStreamingLatestValues", c_uint32,
                    [c_int16, c_void_p, c_void_p], doc)
					
doc = """ void *ps5000aStreamingReady
    (
        int16_t   handle,
        int32_t   noOfSamples,
        uint32_t  startIndex,
        int16_t   overflow,
        uint32_t  triggerAt,
        int16_t   triggered,
        int16_t   autoStop,
        void     *pParameter
    );
    define a python function which accepts the correct arguments, and pass it to the constructor of this type.
    """

ps5000a.StreamingReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                         c_int16,
                                                         c_int32,
                                                         c_uint32,
                                                         c_int16,
                                                         c_uint32,
                                                         c_int16,
                                                         c_int16,
                                                         c_void_p)

ps5000a.StreamingReadyType.__doc__ = doc

doc = """void *ps5000aBlockReady
    (
        int16_t    handle,
        PICO_STATUS    status,
        void    *pParameter
    );
    """
    
ps5000a.BlockReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                     c_int16,
                                                     c_int32,
                                                     c_void_p)
                                                     
ps5000a.BlockReadyType.__doc__ = doc

doc = """ PICO_STATUS ps5000aNoOfStreamingValues
    (
        int16_t   handle,
        uint32_t *noOfValues
    ); """
ps5000a.make_symbol("_NoOfStreamingValues", "ps5000aNoOfStreamingValues", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetMaxDownSampleRatio
    (
        int16_t             handle,
        uint32_t            noOfUnaggreatedSamples,
        uint32_t           *maxDownSampleRatio,
        PS5000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex
    ); """
ps5000a.make_symbol("_GetMaxDownSampleRatio", "ps5000aGetMaxDownSampleRatio", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_int32, c_uint32], doc)

doc = """ PICO_STATUS ps5000aGetValues
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS5000a_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        int16_t            *overflow
    ); """
ps5000a.make_symbol("_GetValues", "ps5000aGetValues", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetValuesAsync
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t            noOfSamples,
        uint32_t            downSampleRatio,
        PS5000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        void               *lpDataReady,
        void               *pParameter
    ); """
ps5000a.make_symbol("_GetValuesAsync", "ps5000aGetValuesAsync", c_uint32,
                    [c_int16, c_uint32, c_uint32, c_uint32, c_int32, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetValuesBulk
    (
        int16_t             handle,
        uint32_t           *noOfSamples,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex,
        uint32_t            downSampleRatio,
        PS5000A_RATIO_MODE  downSampleRatioMode,
        int16_t            *overflow
    ); """
ps5000a.make_symbol("_GetValuesBulk", "ps5000aGetValuesBulk", c_uint32,
                    [c_int16, c_void_p, c_uint32, c_uint32, c_uint32, c_int32, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetValuesOverlapped
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS5000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        int16_t            *overflow
    ); """
ps5000a.make_symbol("_GetValuesOverlapped", "ps5000aGetValuesOverlapped", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetValuesOverlappedBulk
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS5000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex,
        int16_t            *overflow
    ); """
ps5000a.make_symbol("_GetValuesOverlappedBulk", "ps5000aGetValuesOverlappedBulk", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps5000aTriggerWithinPreTriggerSamples
    (
        int16_t                             handle,
        PS5000A_TRIGGER_WITHIN_PRE_TRIGGER  state
    ); """
ps5000a.make_symbol("_TriggerWithinPreTriggerSamples", "ps5000aTriggerWithinPreTriggerSamples", c_uint32,
                    [c_int16, c_int32], doc)

doc = """ PICO_STATUS ps5000aGetTriggerInfoBulk
    (
        int16_t               handle,
        PS5000A_TRIGGER_INFO *triggerInfo,
        uint32_t              fromSegmentIndex,
        uint32_t              toSegmentIndex
    ); """
ps5000a.make_symbol("_GetTriggerInfoBulk", "ps5000aGetTriggerInfoBulk", c_uint32,
                    [c_int16, c_void_p, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps5000aEnumerateUnits
    (
        int16_t *count,
        int8_t  *serials,
        int16_t *serialLth
    ); """
ps5000a.make_symbol("_EnumerateUnits", "ps5000aEnumerateUnits", c_uint32, [c_void_p, c_char_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetChannelInformation
    (
        int16_t               handle,
        PS5000A_CHANNEL_INFO  info,
        int32_t               probe,
        int32_t              *ranges,
        int32_t              *length,
        int32_t               channels
    ); """
ps5000a.make_symbol("_GetChannelInformation", "ps5000aGetChannelInformation", c_uint32,
                    [c_int16, c_int32, c_int32, c_void_p, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps5000aMaximumValue
    (
        int16_t  handle,
        int16_t *value
    ); """
ps5000a.make_symbol("_MaximumValue", "ps5000aMaximumValue", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aMinimumValue
    (
        int16_t  handle,
        int16_t *value
    ); """
ps5000a.make_symbol("_MinimumValue", "ps5000aMinimumValue", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetAnalogueOffset
    (
        int16_t           handle,
        PS5000A_RANGE     range,
        PS5000A_COUPLING  coupling,
        float            *maximumVoltage,
        float            *minimumVoltage
    ); """
ps5000a.make_symbol("_GetAnalogueOffset", "ps5000aGetAnalogueOffset", c_uint32,
                    [c_int16, c_int32, c_int32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetMaxSegments
    (
        int16_t   handle,
        uint32_t *maxSegments
    ); """
ps5000a.make_symbol("_GetMaxSegments", "ps5000aGetMaxSegments", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aChangePowerSource
    (
        int16_t     handle,
        PICO_STATUS powerState
    ); """
ps5000a.make_symbol("_ChangePowerSource", "ps5000aChangePowerSource", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps5000aCurrentPowerSource
    (
        int16_t handle
    ); """
ps5000a.make_symbol("_CurrentPowerSource", "ps5000aCurrentPowerSource", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps5000aStop
    (
        int16_t  handle
    ); """
ps5000a.make_symbol("_Stop", "ps5000aStop", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps5000aPingUnit
    (
        int16_t  handle
    ); """
ps5000a.make_symbol("_PingUnit", "ps5000aPingUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps5000aSetNoOfCaptures
    (
        int16_t   handle,
        uint32_t  nCaptures
    ); """
ps5000a.make_symbol("_SetNoOfCaptures", "ps5000aSetNoOfCaptures", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps5000aGetNoOfCaptures
    (
        int16_t   handle,
        uint32_t *nCaptures
    ); """
ps5000a.make_symbol("_GetNoOfCaptures", "ps5000aGetNoOfCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aGetNoOfProcessedCaptures
    (
         int16_t   handle,
         uint32_t *nProcessedCaptures
    ); """
ps5000a.make_symbol("_GetNoOfProcessedCaptures", "ps5000aGetNoOfProcessedCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aSetDeviceResolution
    (
      int16_t                    handle,
      PS5000A_DEVICE_RESOLUTION  resolution
    ); """
ps5000a.make_symbol("_SetDeviceResolution", "ps5000aSetDeviceResolution", c_uint32, [c_int16, c_int32], doc)

doc = """ PICO_STATUS ps5000aGetDeviceResolution
    (
        int16_t                    handle,
        PS5000A_DEVICE_RESOLUTION *resolution
    ); """
ps5000a.make_symbol("_GetDeviceResolution", "ps5000aGetDeviceResolution", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps5000aSetDigitalPort
    (
        int16_t                 handle,
        PS5000A_CHANNEL         port,
        int16_t                 enabled,
        int16_t                 logicLevel
    ); """
ps5000a.make_symbol("_SetDigitalPort", "ps5000aSetDigitalPort", c_uint32, [c_int16, c_int32, c_int16, c_int16], doc)

doc = """ PICO_STATUS ps5000aSetPulseWidthDigitalPortProperties
    (
        int16_t                             handle,
        PS5000A_DIGITAL_CHANNEL_DIRECTIONS *directions,
        int16_t                             nDirections
    ); """
ps5000a.make_symbol("_SetPulseWidthDigitalPortProperties", "ps5000aSetPulseWidthDigitalPortProperties", c_uint32,
                    [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps5000aSetTriggerDigitalPortProperties
    (
        int16_t                             handle,
        PS5000A_DIGITAL_CHANNEL_DIRECTIONS *directions,
        int16_t                             nDirections
    ); """
ps5000a.make_symbol("_SetTriggerDigitalPortProperties", "ps5000aSetTriggerDigitalPortProperties", c_uint32,
                    [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps5000aSetPulseWidthQualifierProperties
    (
        int16_t                            handle,
        uint32_t                           lower,
        uint32_t                           upper,
        PS5000A_PULSE_WIDTH_TYPE           type
    ); """
ps5000a.make_symbol("_SetPulseWidthQualifierProperties", "ps5000aSetPulseWidthQualifierProperties", c_uint32, 
                    [c_int16, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps5000aSetPulseWidthQualifierConditions
    (
        int16_t                            handle,
        PS5000A_CONDITION                  *conditions,
        int16_t                            nConditions,
        PS5000A_CONDITIONS_INFO            info
    ); """
ps5000a.make_symbol("_SetPulseWidthQualifierConditions", "ps5000aSetPulseWidthQualifierConditions", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int32], doc)
                    
doc = """ PICO_STATUS ps5000aSetPulseWidthQualifierDirections
    (
        int16_t                             handle,
        PS5000A_DIRECTION                   *directions,
        int16_t                             nDirections
    ); """
ps5000a.make_symbol("_SetPulseWidthQualifierDirections", "ps5000aSetPulseWidthQualifierDirections", c_uint32,
                    [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps5000aGetMinimumTimebaseStateless
    (
        int16_t                      handle,
        PS5000A_CHANNEL_FLAGS        enabledChannelOrPortFlags,
        uint32_t                     *timebase,
        double                       *timeInterval,
        PS5000A_DEVICE_RESOLUTION    resolution
    ); """
ps5000a.make_symbol("_GetMinimumTimebaseStateless", "ps5000aGetMinimumTimebaseStateless", c_uint32, [c_int16, c_uint32, c_void_p, c_void_p, c_uint32], doc)    


#
# Copyright (C) 2015-2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the ps4000Api.h C header
file for PicoScope 4000 Series oscilloscopes using the ps4000 driver API
functions.
"""

from ctypes import *
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.library import Library
from picosdk.constants import make_enum


class Ps4000lib(Library):
    def __init__(self):
        super(Ps4000lib, self).__init__("ps4000")


ps4000 = Ps4000lib()

# This field is passed to the driver as a boolean, not an enum.
ps4000.PICO_COUPLING = {
    'AC': 0,
    'DC': 1
}

# A tuple in an enum like this is 2 names for the same value.
ps4000.PS4000_CHANNEL = make_enum([
    "PS4000_CHANNEL_A",
    "PS4000_CHANNEL_B",
    "PS4000_CHANNEL_C",
    "PS4000_CHANNEL_D",
    ("PS4000_EXTERNAL", "PS4000_MAX_CHANNELS"),
    "PS4000_TRIGGER_AUX",
    "PS4000_MAX_TRIGGER_SOURCE",
])

# only include the normal analog channels for now:
ps4000.PICO_CHANNEL = {k[-1]: v for k, v in ps4000.PS4000_CHANNEL.items() if "PS4000_CHANNEL_" in k}

ps4000.PS4000_RANGE = make_enum([
    "PS4000_10MV",
    "PS4000_20MV",
    "PS4000_50MV",
    "PS4000_100MV",
    "PS4000_200MV",
    "PS4000_500MV",
    "PS4000_1V",
    "PS4000_2V",
    "PS4000_5V",
    "PS4000_10V",
    "PS4000_20V",
    "PS4000_50V",
    "PS4000_100V",
    ("PS4000_RESISTANCE_100R", "PS4000_MAX_RANGES"),
    "PS4000_RESISTANCE_1K",
    "PS4000_RESISTANCE_10K",
    "PS4000_RESISTANCE_100K",
    "PS4000_RESISTANCE_1M",
    ("PS4000_ACCELEROMETER_10MV", "PS4000_MAX_RESISTANCES"),
    "PS4000_ACCELEROMETER_20MV",
    "PS4000_ACCELEROMETER_50MV",
    "PS4000_ACCELEROMETER_100MV",
    "PS4000_ACCELEROMETER_200MV",
    "PS4000_ACCELEROMETER_500MV",
    "PS4000_ACCELEROMETER_1V",
    "PS4000_ACCELEROMETER_2V",
    "PS4000_ACCELEROMETER_5V",
    "PS4000_ACCELEROMETER_10V",
    "PS4000_ACCELEROMETER_20V",
    "PS4000_ACCELEROMETER_50V",
    "PS4000_ACCELEROMETER_100V",
    ("PS4000_TEMPERATURE_UPTO_40", "PS4000_MAX_ACCELEROMETER"),
    "PS4000_TEMPERATURE_UPTO_70",
    "PS4000_TEMPERATURE_UPTO_100",
    "PS4000_TEMPERATURE_UPTO_130",
    ("PS4000_RESISTANCE_5K", "PS4000_MAX_TEMPERATURES"),
    "PS4000_RESISTANCE_25K",
    "PS4000_RESISTANCE_50K",
    "PS4000_MAX_EXTRA_RESISTANCES",
])


def process_enum(enum):
    """The PS4000 range enum is complicated enough that we need some clearer logic:"""
    import re
    pattern = re.compile(r'PS4000_([0-9]+M?)V')

    voltage_range = {}

    for enum_item_name, enum_item_value in enum.items():
        match = pattern.match(enum_item_name)
        if match is None:
            continue
        voltage_string = match.group(1)
        voltage = float(voltage_string) if voltage_string[-1] != 'M' else (0.001 * float(voltage_string[:-1]))

        voltage_range[enum_item_value] = voltage

    return voltage_range


ps4000.PICO_VOLTAGE_RANGE = process_enum(ps4000.PS4000_RANGE)

ps4000.PS4000_TIME_UNITS = make_enum([
    'PS4000_FS',
    'PS4000_PS',
    'PS4000_NS',
    'PS4000_US',
    'PS4000_MS',
    'PS4000_S',
    'PS4000_MAX_TIME_UNITS',
])

doc = """ PICO_STATUS ps4000OpenUnit
    (
        int16_t *handle
    ); """
ps4000.make_symbol("_OpenUnit0", "ps4000OpenUnit", c_uint32, [c_void_p, ], doc)

doc = """ PICO_STATUS ps4000OpenUnitAsync
    (
        int16_t *status
    ); """
ps4000.make_symbol("_OpenUnitAsync0", "ps4000OpenUnitAsync", c_uint32, [c_void_p], doc)

doc = """ PICO_STATUS ps4000OpenUnitEx
    (
        int16_t *handle,
        int8_t  *serial
    ); """
ps4000.make_symbol("_OpenUnit", "ps4000OpenUnitEx", c_uint32, [c_void_p, c_char_p], doc)

doc = """ PICO_STATUS ps4000OpenUnitAsyncEx
    (
        int16_t *status,
        int8_t  *serial
    ); """
ps4000.make_symbol("_OpenUnitAsync", "ps4000OpenUnitAsync", c_uint32, [c_void_p, c_char_p], doc)

doc = """ PICO_STATUS ps4000OpenUnitProgress
    (
        int16_t *handle,
        int16_t *progressPercent,
        int16_t *complete
    ); """
ps4000.make_symbol("_OpenUnitProgress", "ps4000OpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000GetUnitInfo
    (
        int16_t    handle,
        int8_t    *string,
        int16_t    stringLength,
        int16_t   *requiredSize,
        PICO_INFO  info
    ); """
ps4000.make_symbol("_GetUnitInfo", "ps4000GetUnitInfo", c_uint32, [c_int16, c_char_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps4000FlashLed
    (
        int16_t  handle,
        int16_t  start
    ); """
ps4000.make_symbol("_FlashLed", "ps4000FlashLed", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps4000IsLedFlashing
    (
        int16_t  handle,
        int16_t *status
    ); """
ps4000.make_symbol("_IsLedFlashing", "ps4000IsLedFlashing", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000CloseUnit
    (
        int16_t  handle
    ); """
ps4000.make_symbol("_CloseUnit", "ps4000CloseUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps4000MemorySegments
    (
        int16_t   handle,
        uint16_t  nSegments,
        int32_t  *nMaxSamples
    ); """
ps4000.make_symbol("_MemorySegments", "ps4000MemorySegments", c_uint32, [c_int16, c_uint16, c_void_p], doc)

doc = """ PICO_STATUS ps4000SetChannel
    (
        int16_t         handle,
        PS4000_CHANNEL  channel,
        int16_t         enabled,
        int16_t         dc,
        PS4000_RANGE    range
    ); """
ps4000.make_symbol("_SetChannel", "ps4000SetChannel", c_uint32, [c_int16, c_int32, c_int16, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps4000SetNoOfCaptures
    (
        int16_t   handle,
        uint16_t  nCaptures
    ); """
ps4000.make_symbol("_SetNoOfCaptures", "ps4000SetNoOfCaptures", c_uint32, [c_int16, c_uint16], doc)

doc = """ PICO_STATUS ps4000GetTimebase
    (
        int16_t   handle,
        uint32_t  timebase,
        int32_t   noSamples,
        int32_t  *timeIntervalNanoseconds,
        int16_t   oversample,
        int32_t  *maxSamples,
        uint16_t  segmentIndex
    ); """
ps4000.make_symbol("_GetTimebase", "ps4000GetTimebase", c_uint32,
                   [c_int16, c_uint32, c_int32, c_void_p, c_int16, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps4000GetTimebase2
    (
        int16_t   handle,
        uint32_t  timebase,
        int32_t   noSamples,
        float    *timeIntervalNanoseconds,
        int16_t   oversample,
        int32_t  *maxSamples,
        uint16_t  segmentIndex
    ); """
ps4000.make_symbol("_GetTimebase2", "ps4000GetTimebase2", c_uint32,
                   [c_int16, c_uint32, c_int32, c_void_p, c_int16, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps4000SigGenOff
    (
        int16_t handle
    ); """
ps4000.make_symbol("_SigGenOff", "ps4000SigGenOff", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps4000SetSigGenArbitrary
    (
        int16_t             handle,
        int32_t             offsetVoltage,
        uint32_t            pkToPk,
        uint32_t            startDeltaPhase,
        uint32_t            stopDeltaPhase,
        uint32_t            deltaPhaseIncrement,
        uint32_t            dwellCount,
        int16_t            *arbitraryWaveform,
        int32_t             arbitraryWaveformSize,
        SWEEP_TYPE          sweepType,
        int16_t             operationType,
        INDEX_MODE          indexMode,
        uint32_t            shots,
        uint32_t            sweeps,
        SIGGEN_TRIG_TYPE    triggerType,
        SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t             extInThreshold
    ); """
ps4000.make_symbol("_SetSigGenArbitrary", "ps4000SetSigGenArbitrary", c_uint32,
                   [c_int16, c_int32, c_uint32, c_uint32, c_uint32, c_uint32, c_uint32, c_void_p,
                    c_int32, c_int32, c_int16, c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps4000SetSigGenBuiltIn
    (
        int16_t             handle,
        int32_t             offsetVoltage,
        uint32_t            pkToPk,
        int16_t             waveType,
        float               startFrequency,
        float               stopFrequency,
        float               increment,
        float               dwellTime,
        SWEEP_TYPE          sweepType,
        int16_t             operationType,
        uint32_t            shots,
        uint32_t            sweeps,
        SIGGEN_TRIG_TYPE    triggerType,
        SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t             extInThreshold
    ); """
ps4000.make_symbol("_SetSigGenBuiltIn", "ps4000SetSigGenBuiltIn", c_uint32,
                   [c_int16, c_int32, c_uint32, c_int16, c_float, c_float, c_float, c_float,
                    c_int32, c_int16, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps4000SigGenFrequencyToPhase
    (
        int16_t     handle,
        double      frequency,
        INDEX_MODE  indexMode,
        uint32_t    bufferLength,
        uint32_t   *phase
    ); """
ps4000.make_symbol("_SigGenFrequencyToPhase", "ps4000SigGenFrequencyToPhase", c_uint32,
                   [c_int16, c_double, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps4000SigGenArbitraryMinMaxValues
    (
        int16_t   handle,
        int16_t  *minArbitraryWaveformValue,
        int16_t  *maxArbitraryWaveformValue,
        uint32_t *minArbitraryWaveformSize,
        uint32_t *maxArbitraryWaveformSize
    ); """
ps4000.make_symbol("_SigGenArbitraryMinMaxValues", "ps4000SigGenArbitraryMinMaxValues", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000SigGenSoftwareControl
    (
        int16_t  handle,
        int16_t  state
    ); """
ps4000.make_symbol("_SigGenSoftwareControl", "ps4000SigGenSoftwareControl", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps4000SetEts
    (
        int16_t          handle,
        PS4000_ETS_MODE  mode,
        int16_t          etsCycles,
        int16_t          etsInterleave,
        int32_t         *sampleTimePicoseconds
    ); """
ps4000.make_symbol("_SetEts", "ps4000SetEts", c_uint32, [c_int16, c_int32, c_int16, c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000SetSimpleTrigger
    (
        int16_t              handle,
        int16_t              enable,
        PS4000_CHANNEL       source,
        int16_t              threshold,
        THRESHOLD_DIRECTION  direction,
        uint32_t             delay,
        int16_t              autoTrigger_ms
    ); """
ps4000.make_symbol("_SetSimpleTrigger", "ps4000SetSimpleTrigger", c_uint32,
                   [c_int16, c_int16, c_int32, c_int16, c_int32, c_uint32, c_int16], doc)

doc = """ PICO_STATUS ps4000SetTriggerChannelProperties
    (
        int16_t                     handle,
        TRIGGER_CHANNEL_PROPERTIES *channelProperties,
        int16_t                     nChannelProperties,
        int16_t                     auxOutputEnable,
        int32_t                     autoTriggerMilliseconds
    ); """
ps4000.make_symbol("_SetTriggerChannelProperties", "ps4000SetTriggerChannelProperties", c_uint32,
                   [c_int16, c_void_p, c_int16, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps4000SetExtTriggerRange
    (
        int16_t       handle,
        PS4000_RANGE  extRange
    ); """
ps4000.make_symbol("_SetExtTriggerRange", "ps4000SetExtTriggerRange", c_uint32, [c_int16, c_int32], doc)

doc = """ PICO_STATUS ps4000SetTriggerChannelConditions
    (
        int16_t             handle,
        TRIGGER_CONDITIONS *conditions,
        int16_t             nConditions
    ); """
ps4000.make_symbol("_SetTriggerChannelConditions", "ps4000SetTriggerChannelConditions", c_uint32,
                   [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps4000SetTriggerChannelDirections
    (
        int16_t              handle,
        THRESHOLD_DIRECTION  channelA,
        THRESHOLD_DIRECTION  channelB,
        THRESHOLD_DIRECTION  channelC,
        THRESHOLD_DIRECTION  channelD,
        THRESHOLD_DIRECTION  ext,
        THRESHOLD_DIRECTION  aux
    ); """
ps4000.make_symbol("_SetTriggerChannelDirections", "ps4000SetTriggerChannelDirections", c_uint32,
                   [c_int16, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps4000SetTriggerDelay
    (
        int16_t   handle,
        uint32_t  delay
    ); """
ps4000.make_symbol("_SetTriggerDelay", "ps4000SetTriggerDelay", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps4000SetPulseWidthQualifier
    (
        int16_t              handle,
        PWQ_CONDITIONS      *conditions,
        int16_t              nConditions,
        THRESHOLD_DIRECTION  direction,
        uint32_t             lower,
        uint32_t             upper,
        PULSE_WIDTH_TYPE     type
    ); """
ps4000.make_symbol("_SetPulseWidthQualifier", "ps4000SetPulseWidthQualifier", c_uint32,
                   [c_int16, c_void_p, c_int16, c_int32, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps4000IsTriggerOrPulseWidthQualifierEnabled
    (
        int16_t  handle,
        int16_t *triggerEnabled,
        int16_t *pulseWidthQualifierEnabled
    ); """
ps4000.make_symbol("_IsTriggerOrPulseWidthQualifierEnabled", "ps4000IsTriggerOrPulseWidthQualifierEnabled", c_uint32,
                   [c_int16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000GetTriggerTimeOffset
    (
        int16_t            handle,
        uint32_t          *timeUpper,
        uint32_t          *timeLower,
        PS4000_TIME_UNITS *timeUnits,
        uint16_t           segmentIndex
    ); """
ps4000.make_symbol("_GetTriggerTimeOffset0", "ps4000GetTriggerTimeOffset", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps4000GetTriggerChannelTimeOffset
    (
        int16_t            handle,
        uint32_t          *timeUpper,
        uint32_t          *timeLower,
        PS4000_TIME_UNITS *timeUnits,
        uint16_t           segmentIndex,
        PS4000_CHANNEL     channel
    ); """
ps4000.make_symbol("_GetTriggerChannelTimeOffset0", "ps4000GetTriggerChannelTimeOffset", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_uint16, c_int32], doc)

doc = """ PICO_STATUS ps4000GetTriggerTimeOffset64
    (
        int16_t            handle,
        int64_t           *time,
        PS4000_TIME_UNITS *timeUnits,
        uint16_t           segmentIndex
    ); """
ps4000.make_symbol("_GetTriggerTimeOffset", "ps4000GetTriggerTimeOffset64", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_uint16], doc)

doc = """ PICO_STATUS ps4000GetTriggerChannelTimeOffset64
    (
        int16_t            handle,
        int64_t           *time,
        PS4000_TIME_UNITS *timeUnits,
        uint16_t           segmentIndex,
        PS4000_CHANNEL     channel
    ); """
ps4000.make_symbol("_GetTriggerChannelTimeOffset64", "ps4000GetTriggerChannelTimeOffset64", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_uint16, c_int32], doc)

doc = """ PICO_STATUS ps4000GetValuesTriggerTimeOffsetBulk
    (
        int16_t            handle,
        uint32_t          *timesUpper,
        uint32_t          *timesLower,
        PS4000_TIME_UNITS *timeUnits,
        uint16_t           fromSegmentIndex,
        uint16_t           toSegmentIndex
    ); """
ps4000.make_symbol("_GetValuesTriggerTimeOffsetBulk0", "ps4000GetValuesTriggerTimeOffsetBulk", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_uint16, c_uint16], doc)

doc = """ PICO_STATUS ps4000GetValuesTriggerChannelTimeOffsetBulk
    (
        int16_t            handle,
        uint32_t          *timesUpper,
        uint32_t          *timesLower,
        PS4000_TIME_UNITS *timeUnits,
        uint16_t           fromSegmentIndex,
        uint16_t           toSegmentIndex,
        PS4000_CHANNEL     channel
    ); """
ps4000.make_symbol("_GetValuesTriggerChannelTimeOffsetBulk0", "ps4000GetValuesTriggerChannelTimeOffsetBulk", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_void_p, c_uint16, c_uint16, c_int32], doc)

doc = """ PICO_STATUS ps4000GetValuesTriggerTimeOffsetBulk64
    (
        int16_t            handle,
        int64_t           *times,
        PS4000_TIME_UNITS *timeUnits,
        uint16_t           fromSegmentIndex,
        uint16_t           toSegmentIndex
    ); """
ps4000.make_symbol("_GetValuesTriggerTimeOffsetBulk", "ps4000GetValuesTriggerTimeOffsetBulk64", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_uint16, c_uint16], doc)

doc = """ PICO_STATUS ps4000GetValuesTriggerChannelTimeOffsetBulk64
    (
        int16_t            handle,
        int64_t           *times,
        PS4000_TIME_UNITS *timeUnits,
        uint16_t           fromSegmentIndex,
        uint16_t           toSegmentIndex,
        PS4000_CHANNEL     channel
    ); """
ps4000.make_symbol("_GetValuesTriggerChannelTimeOffsetBulk", "ps4000GetValuesTriggerChannelTimeOffsetBulk64", c_uint32,
                   [c_int16, c_void_p, c_void_p, c_uint16, c_uint16, c_int32], doc)

doc = """ PICO_STATUS ps4000SetDataBufferBulk
    (
        int16_t         handle,
        PS4000_CHANNEL  channel,
        int16_t        *buffer,
        int32_t         bufferLth,
        uint16_t        waveform
    ); """
ps4000.make_symbol("_SetDataBufferBulk", "ps4000SetDataBufferBulk", c_uint32,
                   [c_int16, c_int32, c_void_p, c_int32, c_uint16], doc)

doc = """ PICO_STATUS ps4000SetDataBuffers
    (
        int16_t         handle,
        PS4000_CHANNEL  channel,
        int16_t        *bufferMax,
        int16_t        *bufferMin,
        int32_t         bufferLth
    ); """
ps4000.make_symbol("_SetDataBuffers", "ps4000SetDataBuffers", c_uint32, [c_int16, c_int32, c_void_p, c_void_p, c_int32],
                   doc)

doc = """ PICO_STATUS ps4000SetDataBufferWithMode
    (
        int16_t         handle,
        PS4000_CHANNEL  channel,
        int16_t        *buffer,
        int32_t         bufferLth,
        RATIO_MODE      mode
    ); """
ps4000.make_symbol("_SetDataBufferWithMode", "ps4000SetDataBufferWithMode", c_uint32,
                   [c_int16, c_int32, c_void_p, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps4000SetDataBuffersWithMode
    (
        int16_t         handle,
        PS4000_CHANNEL  channel,
        int16_t        *bufferMax,
        int16_t        *bufferMin,
        int32_t         bufferLth,
        RATIO_MODE      mode
    ); """
ps4000.make_symbol("_SetDataBuffersWithMode", "ps4000SetDataBuffersWithMode", c_uint32,
                   [c_int16, c_int32, c_void_p, c_void_p, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps4000SetDataBuffer
    (
        int16_t         handle,
        PS4000_CHANNEL  channel,
        int16_t        *buffer,
        int32_t         bufferLth
    ); """
ps4000.make_symbol("_SetDataBuffer", "ps4000SetDataBuffer", c_uint32, [c_int16, c_int32, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps4000SetEtsTimeBuffer
    (
        int16_t  handle,
        int64_t *buffer,
        int32_t  bufferLth
    ); """
ps4000.make_symbol("_SetEtsTimeBuffer", "ps4000SetEtsTimeBuffer", c_uint32, [c_int16, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps4000SetEtsTimeBuffers
    (
        int16_t   handle,
        uint32_t *timeUpper,
        uint32_t *timeLower,
        int32_t   bufferLth
    ); """
ps4000.make_symbol("_SetEtsTimeBuffers", "ps4000SetEtsTimeBuffers", c_uint32, [c_int16, c_void_p, c_void_p, c_int32],
                   doc)

doc = """ PICO_STATUS ps4000RunBlock
    (
        int16_t           handle,
        int32_t           noOfPreTriggerSamples,
        int32_t           noOfPostTriggerSamples,
        uint32_t          timebase,
        int16_t           oversample,
        int32_t          *timeIndisposedMs,
        uint16_t          segmentIndex,
        ps4000BlockReady  lpReady,
        void             *pParameter
    ); """
ps4000.make_symbol("_RunBlock", "ps4000RunBlock", c_uint32,
                   [c_int16, c_int32, c_int32, c_uint32, c_int16, c_void_p, c_uint16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000RunStreaming
    (
        int16_t            handle,
        uint32_t          *sampleInterval,
        PS4000_TIME_UNITS  sampleIntervalTimeUnits,
        uint32_t           maxPreTriggerSamples,
        uint32_t           maxPostPreTriggerSamples,
        int16_t            autoStop,
        uint32_t           downSampleRatio,
        uint32_t           overviewBufferSize
    ); """
ps4000.make_symbol("_RunStreaming", "ps4000RunStreaming", c_uint32,
                   [c_int16, c_void_p, c_int32, c_uint32, c_uint32, c_int16, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps4000RunStreamingEx
    (
        int16_t            handle,
        uint32_t          *sampleInterval,
        PS4000_TIME_UNITS  sampleIntervalTimeUnits,
        uint32_t           maxPreTriggerSamples,
        uint32_t           maxPostPreTriggerSamples,
        int16_t            autoStop,
        uint32_t           downSampleRatio,
        int16_t            downSampleRatioMode,
        uint32_t           overviewBufferSize
    ); """
ps4000.make_symbol("_RunStreamingEx", "ps4000RunStreamingEx", c_uint32,
                   [c_int16, c_void_p, c_int32, c_uint32, c_uint32, c_int16, c_uint32, c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps4000IsReady
    (
        int16_t handle,
        int16_t * ready
    ); """
ps4000.make_symbol("_IsReady", "ps4000IsReady", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000GetStreamingLatestValues
    (
        int16_t               handle,
        ps4000StreamingReady  lpPs4000Ready,
        void                 *pParameter
    ); """
ps4000.make_symbol("_GetStreamingLatestValues", "ps4000GetStreamingLatestValues", c_uint32,
                   [c_int16, c_void_p, c_void_p], doc)
				   
doc = """ void *ps4000StreamingReady
    (
        int16_t   handle,
        int32_t   noOfSamples,
        uint32_t  startIndex,
        int16_t   overflow,
        uint32_t  triggerAt,
        int16_t   triggered,
        int16_t   autoStop,
        void     *pParameter
    );
    define a python function which accepts the correct arguments, and pass it to the constructor of this type.
    """

ps4000.StreamingReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                         c_int16,
                                                         c_int32,
                                                         c_uint32,
                                                         c_int16,
                                                         c_uint32,
                                                         c_int16,
                                                         c_int16,
                                                         c_void_p)

ps4000.StreamingReadyType.__doc__ = doc

doc = """ PICO_STATUS ps4000NoOfStreamingValues
    (
        int16_t   handle,
        uint32_t *noOfValues
    ); """
ps4000.make_symbol("_NoOfStreamingValues", "ps4000NoOfStreamingValues", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000GetMaxDownSampleRatio
    (
        int16_t   handle,
        uint32_t  noOfUnaggreatedSamples,
        uint32_t *maxDownSampleRatio,
        int16_t   downSampleRatioMode,
        uint16_t  segmentIndex
    ); """
ps4000.make_symbol("_GetMaxDownSampleRatio", "ps4000GetMaxDownSampleRatio", c_uint32,
                   [c_int16, c_uint32, c_void_p, c_int16, c_uint16], doc)

doc = """ PICO_STATUS ps4000GetValues
    (
        int16_t   handle,
        uint32_t  startIndex,
        uint32_t *noOfSamples,
        uint32_t  downSampleRatio,
        int16_t   downSampleRatioMode,
        uint16_t  segmentIndex,
        int16_t  *overflow
    ); """
ps4000.make_symbol("_GetValues", "ps4000GetValues", c_uint32,
                   [c_int16, c_uint32, c_void_p, c_uint32, c_int16, c_uint16, c_void_p], doc)

doc = """ PICO_STATUS ps4000GetValuesBulk
    (
        int16_t   handle,
        uint32_t *noOfSamples,
        uint16_t  fromSegmentIndex,
        uint16_t  toSegmentIndex,
        int16_t  *overflow
    ); """
ps4000.make_symbol("_GetValuesBulk", "ps4000GetValuesBulk", c_uint32, [c_int16, c_void_p, c_uint16, c_uint16, c_void_p],
                   doc)

doc = """ PICO_STATUS ps4000GetValuesAsync
    (
        int16_t   handle,
        uint32_t  startIndex,
        uint32_t  noOfSamples,
        uint32_t  downSampleRatio,
        int16_t   downSampleRatioMode,
        uint16_t  segmentIndex,
        void     *lpDataReady,
        void     *pParameter
    ); """
ps4000.make_symbol("_GetValuesAsync", "ps4000GetValuesAsync", c_uint32,
                   [c_int16, c_uint32, c_uint32, c_uint32, c_int16, c_uint16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000Stop
    (
        int16_t  handle
    ); """
ps4000.make_symbol("_Stop", "ps4000Stop", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps4000SetProbe
    (
        int16_t       handle,
        PS4000_PROBE  probe,
        PS4000_RANGE  range
    ); """
ps4000.make_symbol("_SetProbe", "ps4000SetProbe", c_uint32, [c_int16, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps4000HoldOff
    (
        int16_t              handle,
        uint64_t             holdoff,
        PS4000_HOLDOFF_TYPE  type
    ); """
ps4000.make_symbol("_HoldOff", "ps4000HoldOff", c_uint32, [c_int16, c_uint64, c_int32], doc)

doc = """ PICO_STATUS ps4000GetProbe
    (
        int16_t       handle,
        PS4000_PROBE *probe
    ); """
ps4000.make_symbol("_GetProbe", "ps4000GetProbe", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps4000GetChannelInformation
    (
        int16_t              handle,
        PS4000_CHANNEL_INFO  info,
        int32_t              probe,
        int32_t             *ranges,
        int32_t             *length,
        int32_t              channels
    ); """
ps4000.make_symbol("_GetChannelInformation", "ps4000GetChannelInformation", c_uint32,
                   [c_int16, c_int32, c_int32, c_void_p, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps4000SetFrequencyCounter
    (
        int16_t                         handle,
        PS4000_CHANNEL                  channel,
        int16_t                         enabled,
        PS4000_FREQUENCY_COUNTER_RANGE  range,
        int16_t                         thresholdMajor,
        int16_t                         thresholdMinor
    ); """
ps4000.make_symbol("_SetFrequencyCounter", "ps4000SetFrequencyCounter", c_uint32,
                   [c_int16, c_int32, c_int16, c_int32, c_int16, c_int16], doc)

doc = """ PICO_STATUS ps4000EnumerateUnits
    (
        int16_t *count,
        int8_t  *serials,
        int16_t *serialLth
    ); """
ps4000.make_symbol("_EnumerateUnits", "ps4000EnumerateUnits", c_uint32, [c_void_p, c_char_p, c_void_p], doc)

doc = """ PICO_STATUS ps4000PingUnit
    (
        int16_t  handle
    ); """
ps4000.make_symbol("_PingUnit", "ps4000PingUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps4000SetBwFilter
    (
        int16_t         handle,
        PS4000_CHANNEL  channel,
        int16_t         enable
    ); """
ps4000.make_symbol("_SetBwFilter", "ps4000SetBwFilter", c_uint32, [c_int16, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps4000TriggerWithinPreTriggerSamples
    (
        int16_t  handle,
        int16_t  state
    ); """
ps4000.make_symbol("_TriggerWithinPreTriggerSamples", "ps4000TriggerWithinPreTriggerSamples", c_uint32,
                   [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps4000GetNoOfCaptures
    (
        int16_t   handle,
        uint16_t *nCaptures
    ); """
ps4000.make_symbol("_GetNoOfCaptures", "ps4000GetNoOfCaptures", c_uint32, [c_int16, c_void_p], doc)



#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
from __future__ import division
import numpy as np
from picosdk.constants import PICO_STATUS, PICO_STATUS_LOOKUP
from picosdk.errors import PicoSDKCtypesError


def adc2mV(bufferADC, range, maxADC):
    """ 
        adc2mc(
                c_short_Array           bufferADC
                int                     range
                c_int32                 maxADC
                )
               
        Takes a buffer of raw adc count values and converts it into millivolts
    """

    channelInputRanges = [10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000, 100000, 200000]
    vRange = channelInputRanges[range]
    bufferV = [(x * vRange) / maxADC.value for x in bufferADC]

    return bufferV
	
def adc2mVpl1000(bufferADC, range, maxADC):
	"""
		adc2mVpl1000(
						c_short_Array		bufferADC,
						int 				range,
						c_int32				maxADC
						)
		
		Takes a buffer of raw adc count values and converts it into millvolts
	"""
	
	bufferV = [(x * range) / maxADC.value for x in bufferADC]
	
	return bufferV

def mV2adc(millivolts, range, maxADC):
    """
        mV2adc(
                float                   millivolts
				int                     range
                c_int32                 maxADC
                )
        Takes a voltage value and converts it into adc counts
    """
    channelInputRanges = [10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000, 100000, 200000]
    vRange = channelInputRanges[range]
    adcValue = round((millivolts * maxADC.value)/vRange)

    return adcValue

def mV2adcpl1000(millivolts, range, maxADC):
	"""
		mV2adc(
				float				millivolts,
				int					range,
				c_int32				maxADC
				)
		Takes a voltage value and converts it to adc counts
	"""
	adcValue = round((millivolts * maxADC.value)/range)
	
	return adcValue


def splitMSOData(dataLength, data):
    """
    This method converts an array of values for a ditial port into the binary equivalent, splitting the bits by
    digital channel.

    Returns a set of 8 variables, each of which corresponds to the binary data values over time of the different
    digital channels from the lowest significant bit to the most significant bit. For PORT0 this will be in the order
    (D0, D1, D2, ... D7) and for PORT1 this will be (D8, D9, D10, ... D15).

        splitMSOData(
                        c_int32         dataLength
                        c_int16 array   data
                        )
    """
    # Makes an array for each digital channel
    binaryBufferD0 = np.chararray((dataLength.value, 1))
    binaryBufferD1 = np.chararray((dataLength.value, 1))
    binaryBufferD2 = np.chararray((dataLength.value, 1))
    binaryBufferD3 = np.chararray((dataLength.value, 1))
    binaryBufferD4 = np.chararray((dataLength.value, 1))
    binaryBufferD7 = np.chararray((dataLength.value, 1))
    binaryBufferD5 = np.chararray((dataLength.value, 1))
    binaryBufferD6 = np.chararray((dataLength.value, 1))
    
    # Changes the data from int type to a binary type and then separates the data for each digital channel
    for i in range(0, dataLength.value):
        MSOData = data[i]
        binaryMSOData = bin(MSOData)
        binaryMSOData = binaryMSOData[2:]
        binaryMSOData = binaryMSOData.zfill(8)
        binaryBufferD0[i] = binaryMSOData[7]
        binaryBufferD1[i] = binaryMSOData[6]
        binaryBufferD2[i] = binaryMSOData[5]
        binaryBufferD3[i] = binaryMSOData[4]
        binaryBufferD4[i] = binaryMSOData[3]
        binaryBufferD5[i] = binaryMSOData[2]
        binaryBufferD6[i] = binaryMSOData[1]
        binaryBufferD7[i] = binaryMSOData[0]

    return binaryBufferD0, \
           binaryBufferD1, \
           binaryBufferD2, \
           binaryBufferD3, \
           binaryBufferD4, \
           binaryBufferD5, \
           binaryBufferD6, \
           binaryBufferD7


def splitMSODataFast(dataLength, data):
    """
    # This implementation will work on either channel in the same way as the splitMSOData method above, albeit in a
    more efficient manner.

    Returns a tuple of 8 arrays, each of which is the values over time of a different digital channel.
    The tuple contains the channels in order (D7, D6, D5, ... D0) or equivalently (D15, D14, D13, ... D8).

        splitMSODataFast(
                        c_int32         dataLength
                        c_int16 array   data
                        )
    """
    # Makes an array for each digital channel
    bufferBinaryDj = (
        np.chararray(dataLength.value),
        np.chararray(dataLength.value),
        np.chararray(dataLength.value),
        np.chararray(dataLength.value),
        np.chararray(dataLength.value),
        np.chararray(dataLength.value),
        np.chararray(dataLength.value),
        np.chararray(dataLength.value),
    )
    # Splits out the individual bits from the port into the binary values for each digital channel/pin.
    for i in range(dataLength.value):
        for j in range(8):
            bufferBinaryDj[j][i] = 1 if (data[i] & (1 << (7-j))) else 0

    return bufferBinaryDj


def assert_pico_ok(status):
    """
        assert_pico_ok(
                        status
                       )
    """
    # checks for PICO_OK status return
    if status != PICO_STATUS['PICO_OK']:
        raise PicoSDKCtypesError("PicoSDK returned '{}'".format(PICO_STATUS_LOOKUP[status]))


def assert_pico2000_ok(status):
    """
        assert_pico_ok(
                        status
                       )
    """
    # checks for PICO_OK status return
    if status > 0:
        errorCheck = True
    else:
        errorCheck = False
        raise PicoSDKCtypesError("Unsuccessful API call")



#
# Copyright (C) 2022 Pico Technology Ltd. See LICENSE file for terms.
#
# PT104 Example
# This example opens a pt104, sets up a single channel and collects data before closing the pt104

import ctypes
from picosdk.usbPT104 import usbPt104 as pt104
import numpy as np
from picosdk.functions import assert_pico_ok
from time import sleep
import matplotlib.pyplot as plt

# Create chandle and status ready for use
status = {}
chandle = ctypes.c_int16()

# Open the device
status["openUnit"] = pt104.UsbPt104OpenUnit(ctypes.byref(chandle),0)
assert_pico_ok(status["openUnit"])

# Set mains noise filtering
sixty_hertz = 0 #50 Hz
status["setMains"] = pt104.UsbPt104SetMains(chandle, sixty_hertz)
assert_pico_ok(status["setMains"])

# Setup channel 1
channel = pt104.PT104_CHANNELS["USBPT104_CHANNEL_1"] #channel 1
datatype = pt104.PT104_DATA_TYPE["USBPT104_PT100"] #pt100
noOfWires = 4 #wires

status["setChannel1"] = pt104.UsbPt104SetChannel(chandle, channel, datatype, noOfWires)
assert_pico_ok(status["setChannel1"])

#collect data
print("collecting data")
numSamples = 20

data = (ctypes.c_int32 * numSamples)()

for i in range(numSamples):

    #pause
    sleep(2)

    # Get values
    measurement = ctypes.c_int32()
    filtered = 1 # true
    status["getValue"] = pt104.UsbPt104GetValue(chandle, channel, ctypes.byref(measurement), filtered)
    assert_pico_ok(status["getValue"])
    
    data[i] = measurement.value

samples = np.linspace(0, numSamples*2, numSamples)
dataTemp = [x /1000 for x in data]   

plt.plot(samples, dataTemp)
plt.xlabel('Time (s)')
plt.ylabel('Temperature ($^o$C)')
plt.show()

# Close the device
status["closeUnit"] = pt104.UsbPt104CloseUnit(chandle)
assert_pico_ok(status["closeUnit"])

print(status)


#
# Copyright (C) 2014-2018 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the ps2000aApi.h C header
file for PicoScope 2000 Series oscilloscopes using the ps2000a driver API functions.
"""

from ctypes import *
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.library import Library
from picosdk.constants import make_enum


class Ps2000alib(Library):
    def __init__(self):
        super(Ps2000alib, self).__init__("ps2000a")


ps2000a = Ps2000alib()

# A tuple in an enum like this is 2 names for the same value.
ps2000a.PS2000A_CHANNEL = make_enum([
    "PS2000A_CHANNEL_A",
    "PS2000A_CHANNEL_B",
    "PS2000A_CHANNEL_C",
    "PS2000A_CHANNEL_D",
    ("PS2000A_EXTERNAL", "PS2000A_MAX_CHANNELS"),
    "PS2000A_TRIGGER_AUX",
    "PS2000A_MAX_TRIGGER_SOURCE",
])

# only include the normal analog channels for now:
ps2000a.PICO_CHANNEL = {k[-1]: v for k, v in ps2000a.PS2000A_CHANNEL.items() if "PS2000A_CHANNEL_" in k}


ps2000a.PS2000A_COUPLING = make_enum([
    'PS2000A_AC',
    'PS2000A_DC',
])

# Just use AC and DC.
ps2000a.PICO_COUPLING = {k[-2:]: v for k, v in ps2000a.PS2000A_COUPLING.items()}

ps2000a.PS2000A_RANGE = make_enum([
    "PS2000A_10MV",
    "PS2000A_20MV",
    "PS2000A_50MV",
    "PS2000A_100MV",
    "PS2000A_200MV",
    "PS2000A_500MV",
    "PS2000A_1V",
    "PS2000A_2V",
    "PS2000A_5V",
    "PS2000A_10V",
    "PS2000A_20V",
    "PS2000A_50V",
    "PS2000A_MAX_RANGES",
])

ps2000a.PICO_VOLTAGE_RANGE = {
    v: float(k.split('_')[1][:-1]) if k[-2] != 'M' else (0.001 * float(k.split('_')[1][:-2]))
    for k, v in ps2000a.PS2000A_RANGE.items() if k != "PS2000A_MAX_RANGES"
}

ps2000a.MAX_MEMORY = 128e6

ps2000a.PS2000A_RATIO_MODE = {
    'PS2000A_RATIO_MODE_NONE': 0,
    'PS2000A_RATIO_MODE_AGGREGATE': 1,
    'PS2000A_RATIO_MODE_DECIMATE': 2,
    'PS2000A_RATIO_MODE_AVERAGE': 4,
}

ps2000a.PS2000A_TIME_UNITS = make_enum([
    'PS2000A_FS',
    'PS2000A_PS',
    'PS2000A_NS',
    'PS2000A_US',
    'PS2000A_MS',
    'PS2000A_S',
    'PS2000A_MAX_TIME_UNITS',
])

ps2000a.PICO_RATIO_MODE = {k[19:]: v for k, v in ps2000a.PS2000A_RATIO_MODE.items()}


def _define_threshold_direction():
    ps2000a_above = 0
    ps2000a_below = 1
    ps2000a_rising = 2
    ps2000a_falling = 3
    ps2000a_rising_or_falling = 4
    ps2000a_above_lower = 5
    ps2000a_below_lower = 6
    ps2000a_rising_lower = 7
    ps2000a_falling_lower = 8

    ps2000a_inside = ps2000a_above
    ps2000a_outside = ps2000a_below
    ps2000a_enter = ps2000a_rising
    ps2000a_exit = ps2000a_falling
    ps2000a_enter_or_exit = ps2000a_rising_or_falling
    ps2000a_positive_runt = 9
    ps2000a_negative_runt = 10

    ps2000a_none = ps2000a_rising

    return {k.upper(): v for k, v in locals().items() if k.startswith("ps2000a")}


ps2000a.PS2000A_THRESHOLD_DIRECTION = _define_threshold_direction()

ps2000a.PICO_THRESHOLD_DIRECTION = {
    k[8:]: v for k, v in ps2000a.PS2000A_THRESHOLD_DIRECTION.items()
}

ps2000a.PS2000A_THRESHOLD_MODE = make_enum([
    "PS2000A_LEVEL",
    "PS2000A_WINDOW",
])

ps2000a.PS2000A_TRIGGER_STATE = make_enum([
    "PS2000A_CONDITION_DONT_CARE",
    "PS2000A_CONDITION_TRUE",
    "PS2000A_CONDITION_FALSE",
    "PS2000A_CONDITION_MAX",
])


def _define_digital_port():
    ps2000a_digital_port0 = 0x80
    ps2000a_digital_port1 = ps2000a_digital_port0 + 1
    ps2000a_digital_port2 = ps2000a_digital_port0 + 2
    ps2000a_digital_port3 = ps2000a_digital_port0 + 3
    ps2000a_max_digital_ports = ps2000a_digital_port3 - ps2000a_digital_port0 + 1
    
    return {k.upper(): v for k, v in locals().items() if k.startswith("ps2000a")}

ps2000a.PS2000A_DIGITAL_PORT = _define_digital_port()

ps2000a.PS2000A_DIGITAL_CHANNEL = make_enum([
    "PS2000A_DIGITAL_CHANNEL_0",
    "PS2000A_DIGITAL_CHANNEL_1",
    "PS2000A_DIGITAL_CHANNEL_2",
    "PS2000A_DIGITAL_CHANNEL_3",
    "PS2000A_DIGITAL_CHANNEL_4",
    "PS2000A_DIGITAL_CHANNEL_5",
    "PS2000A_DIGITAL_CHANNEL_6",
    "PS2000A_DIGITAL_CHANNEL_7",
    "PS2000A_DIGITAL_CHANNEL_8",
    "PS2000A_DIGITAL_CHANNEL_9",
    "PS2000A_DIGITAL_CHANNEL_10",
    "PS2000A_DIGITAL_CHANNEL_11",
    "PS2000A_DIGITAL_CHANNEL_12",
    "PS2000A_DIGITAL_CHANNEL_13",
    "PS2000A_DIGITAL_CHANNEL_14",
    "PS2000A_DIGITAL_CHANNEL_15",
    "PS2000A_DIGITAL_CHANNEL_16",
    "PS2000A_DIGITAL_CHANNEL_17",
    "PS2000A_DIGITAL_CHANNEL_18",
    "PS2000A_DIGITAL_CHANNEL_19",
    "PS2000A_DIGITAL_CHANNEL_20",
    "PS2000A_DIGITAL_CHANNEL_21",
    "PS2000A_DIGITAL_CHANNEL_22",
    "PS2000A_DIGITAL_CHANNEL_23",
    "PS2000A_DIGITAL_CHANNEL_24",
    "PS2000A_DIGITAL_CHANNEL_25",
    "PS2000A_DIGITAL_CHANNEL_26",
    "PS2000A_DIGITAL_CHANNEL_27",
    "PS2000A_DIGITAL_CHANNEL_28",
    "PS2000A_DIGITAL_CHANNEL_29",
    "PS2000A_DIGITAL_CHANNEL_30",
    "PS2000A_DIGITAL_CHANNEL_31",
    "PS2000A_MAX_DIGITAL_CHANNELS",
])

ps2000a.PS2000A_DIGITAL_DIRECTION = make_enum([
    "PS2000A_DIGITAL_DONT_CARE",
    "PS2000A_DIGITAL_DIRECTION_LOW",
    "PS2000A_DIGITAL_DIRECTION_HIGH",
    "PS2000A_DIGITAL_DIRECTION_RISING",
    "PS2000A_DIGITAL_DIRECTION_FALLING",
    "PS2000A_DIGITAL_DIRECTION_RISING_OR_FALLING",
    "PS2000A_DIGITAL_MAX_DIRECTION",
])


class PS2000A_TRIGGER_CONDITIONS(Structure):
    _pack_ = 1
    _fields_ = [("channelA", c_int32),
                ("channelB", c_int32),
                ("channelC", c_int32),
                ("channelD", c_int32),
                ("external", c_int32),
                ("aux", c_int32),
                ("pulseWidthQualifier", c_int32),
                ("digital", c_int32)]


class PS2000A_PWQ_CONDITIONS(Structure):
    _pack_ = 1
    _fields_ = [("channelA", c_int32),
                ("channelB", c_int32),
                ("channelC", c_int32),
                ("channelD", c_int32),
                ("external", c_int32),
                ("aux", c_int32),
                ("digital", c_int32)]


class PS2000A_DIGITAL_CHANNEL_DIRECTIONS(Structure):
    _pack_ = 1
    _fields_ = [("channel", c_int32),
                ("direction", c_int32)]


class PS2000A_TRIGGER_CHANNEL_PROPERTIES(Structure):
    _pack_ = 1
    _fields_ = [("thresholdUpper", c_int16),
                ("thresholdHysteresis", c_uint16),
                ("thresholdLower", c_int16),
                ("thresholdLowerHysteresis", c_uint16),
                ("channel", c_int32),
                ("thresholdMode", c_int32)]


doc = """ PICO_STATUS ps2000aOpenUnit
    (
        int16_t *status,
        int8_t  *serial
    ); """
ps2000a.make_symbol("_OpenUnit", "ps2000aOpenUnit", c_uint32, [c_void_p, c_char_p], doc)

doc = """ PICO_STATUS ps2000aOpenUnitAsync
    (
        int16_t *status,
        int8_t    *serial
    ); """
ps2000a.make_symbol("_OpenUnitAsync", "ps2000aOpenUnitAsync", c_uint32, [c_void_p, c_char_p], doc)

doc = """ PICO_STATUS ps2000aOpenUnitProgress
    (
        int16_t *handle,
        int16_t *progressPercent,
        int16_t *complete
    ); """
ps2000a.make_symbol("_OpenUnitProgress", "ps2000aOpenUnitProgress", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetUnitInfo
    (
        int16_t   handle,
        int8_t   *string,
        int16_t   stringLength,
        int16_t  *requiredSize,
        PICO_INFO info
    ); """
ps2000a.make_symbol("_GetUnitInfo", "ps2000aGetUnitInfo", c_uint32, [c_int16, c_char_p, c_int16, c_void_p, c_uint32],
                    doc)

doc = """ PICO_STATUS ps2000aFlashLed
    (
        int16_t handle,
        int16_t start
    ); """
ps2000a.make_symbol("_FlashLed", "ps2000aFlashLed", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps2000aCloseUnit
    (
        int16_t handle
    ); """
ps2000a.make_symbol("_CloseUnit", "ps2000aCloseUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps2000aMemorySegments
    (
        int16_t   handle,
        uint32_t  nSegments,
        int32_t  *nMaxSamples
    ); """
ps2000a.make_symbol("_MemorySegments", "ps2000aMemorySegments", c_uint32, [c_int16, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps2000aSetChannel
    (
        int16_t          handle,
        PS2000A_CHANNEL  channel,
        int16_t          enabled,
        PS2000A_COUPLING type,
        PS2000A_RANGE    range,
        float            analogOffset
    ); """
ps2000a.make_symbol("_SetChannel", "ps2000aSetChannel", c_uint32,
                    [c_int16, c_int32, c_int16, c_int32, c_int32, c_float], doc)

doc = """ PICO_STATUS ps2000aSetDigitalPort
    (
        int16_t              handle,
        PS2000A_DIGITAL_PORT port,
        int16_t              enabled,
        int16_t              logicLevel
    ); """
ps2000a.make_symbol("_SetDigitalPort", "ps2000aSetDigitalPort", c_uint32, [c_int16, c_int32, c_int16, c_int16], doc)

doc = """ PICO_STATUS ps2000aSetNoOfCaptures
    (
        int16_t  handle,
        uint32_t nCaptures
    ); """
ps2000a.make_symbol("_SetNoOfCaptures", "ps2000aSetNoOfCaptures", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps2000aGetTimebase
    (
        int16_t  handle,
        uint32_t timebase,
        int32_t  noSamples,
        int32_t *timeIntervalNanoseconds,
        int16_t  oversample,
        int32_t *totalSamples,
        uint32_t segmentIndex
    ); """
ps2000a.make_symbol("_GetTimebase", "ps2000aGetTimebase", c_uint32,
                    [c_int16, c_uint32, c_int32, c_void_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps2000aGetTimebase2
    (
        int16_t  handle,
        uint32_t timebase,
        int32_t  noSamples,
        float   *timeIntervalNanoseconds,
        int16_t  oversample,
        int32_t *totalSamples,
        uint32_t segmentIndex
    ); """
ps2000a.make_symbol("_GetTimebase2", "ps2000aGetTimebase2", c_uint32,
                    [c_int16, c_uint32, c_int32, c_void_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS ps2000aSetSigGenArbitrary
    (
        int16_t                     handle,
        int32_t                     offsetVoltage,
        uint32_t                    pkToPk,
        uint32_t                    startDeltaPhase,
        uint32_t                    stopDeltaPhase,
        uint32_t                    deltaPhaseIncrement,
        uint32_t                    dwellCount,
        int16_t                    *arbitraryWaveform,
        int32_t                     arbitraryWaveformSize,
        PS2000A_SWEEP_TYPE          sweepType,
        PS2000A_EXTRA_OPERATIONS    operation,
        PS2000A_INDEX_MODE          indexMode,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS2000A_SIGGEN_TRIG_TYPE    triggerType,
        PS2000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps2000a.make_symbol("_SetSigGenArbitrary", "ps2000aSetSigGenArbitrary", c_uint32,
                    [c_int16, c_int32, c_uint32, c_uint32, c_uint32, c_uint32, c_uint32, c_void_p, c_int32, c_int32,
                     c_int32,
                     c_int32, c_uint32, c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps2000aSetSigGenBuiltIn
    (
        int16_t                     handle,
        int32_t                     offsetVoltage,
        uint32_t                    pkToPk,
        int16_t                     waveType,
        float                       startFrequency,
        float                       stopFrequency,
        float                       increment,
        float                       dwellTime,
        PS2000A_SWEEP_TYPE          sweepType,
        PS2000A_EXTRA_OPERATIONS    operation,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS2000A_SIGGEN_TRIG_TYPE    triggerType,
        PS2000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps2000a.make_symbol("_SetSigGenBuiltIn", "ps2000aSetSigGenBuiltIn", c_uint32,
                    [c_int16, c_int32, c_uint32, c_int16, c_float, c_float, c_float, c_float, c_int32, c_int32,
                     c_uint32,
                     c_uint32, c_int32, c_int32, c_int16], doc)

doc = """ PICO_STATUS ps2000aSetSigGenPropertiesArbitrary
    (
        int16_t                     handle,
        uint32_t                    startDeltaPhase,
        uint32_t                    stopDeltaPhase,
        uint32_t                    deltaPhaseIncrement,
        uint32_t                    dwellCount,
        PS2000A_SWEEP_TYPE          sweepType,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS2000A_SIGGEN_TRIG_TYPE    triggerType,
        PS2000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps2000a.make_symbol("_SetSigGenPropertiesArbitrary", "ps2000aSetSigGenPropertiesArbitrary", c_uint32,
                    [c_int16, c_uint32, c_uint32, c_uint32, c_uint32, c_int32, c_uint32, c_uint32, c_int32, c_int32,
                     c_int16], doc)

doc = """ PICO_STATUS ps2000aSetSigGenPropertiesBuiltIn
    (
        int16_t                     handle,
        double                      startFrequency,
        double                      stopFrequency,
        double                      increment,
        double                      dwellTime,
        PS2000A_SWEEP_TYPE          sweepType,
        uint32_t                    shots,
        uint32_t                    sweeps,
        PS2000A_SIGGEN_TRIG_TYPE    triggerType,
        PS2000A_SIGGEN_TRIG_SOURCE  triggerSource,
        int16_t                     extInThreshold
    ); """
ps2000a.make_symbol("_SetSigGenPropertiesBuiltIn", "ps2000aSetSigGenPropertiesBuiltIn", c_uint32,
                    [c_int16, c_double, c_double, c_double, c_double, c_int32, c_uint32, c_uint32, c_int32, c_int32,
                     c_int16], doc)

doc = """ PICO_STATUS ps2000aSigGenFrequencyToPhase
    (
        int16_t             handle,
        double              frequency,
        PS2000A_INDEX_MODE  indexMode,
        uint32_t            bufferLength,
        uint32_t           *phase
    ); """
ps2000a.make_symbol("_SigGenFrequencyToPhase", "ps2000aSigGenFrequencyToPhase", c_uint32,
                    [c_int16, c_double, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps2000aSigGenArbitraryMinMaxValues
    (
        int16_t   handle,
        int16_t  *minArbitraryWaveformValue,
        int16_t  *maxArbitraryWaveformValue,
        uint32_t *minArbitraryWaveformSize,
        uint32_t *maxArbitraryWaveformSize
    ); """
ps2000a.make_symbol("_SigGenArbitraryMinMaxValues", "ps2000aSigGenArbitraryMinMaxValues", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps2000aSigGenSoftwareControl
    (
        int16_t  handle,
        int16_t  state
    ); """
ps2000a.make_symbol("_SigGenSoftwareControl", "ps2000aSigGenSoftwareControl", c_uint32, [c_int16, c_int16], doc)

doc = """ PICO_STATUS ps2000aSetEts
    (
        int16_t           handle,
        PS2000A_ETS_MODE  mode,
        int16_t           etsCycles,
        int16_t           etsInterleave,
        int32_t          *sampleTimePicoseconds
    ); """
ps2000a.make_symbol("_SetEts", "ps2000aSetEts", c_uint32, [c_int16, c_int32, c_int16, c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps2000aSetSimpleTrigger
    (
        int16_t                      handle,
        int16_t                      enable,
        PS2000A_CHANNEL              source,
        int16_t                      threshold,
        PS2000A_THRESHOLD_DIRECTION  direction,
        uint32_t                     delay,
        int16_t                      autoTrigger_ms
    ); """
ps2000a.make_symbol("_SetSimpleTrigger", "ps2000aSetSimpleTrigger", c_uint32,
                    [c_int16, c_int16, c_int32, c_int16, c_int32, c_uint32, c_int16], doc)

doc = """ PICO_STATUS ps2000aSetTriggerDigitalPortProperties
    (
        int16_t                             handle,
        PS2000A_DIGITAL_CHANNEL_DIRECTIONS *directions,
        int16_t                             nDirections
    ); """
ps2000a.make_symbol("_SetTriggerDigitalPortProperties", "ps2000aSetTriggerDigitalPortProperties", c_uint32,
                    [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps2000aSetDigitalAnalogTriggerOperand
    (
        int16_t handle,
        PS2000A_TRIGGER_OPERAND operand
    ); """
ps2000a.make_symbol("_SetDigitalAnalogTriggerOperand", "ps2000aSetDigitalAnalogTriggerOperand", c_uint32,
                    [c_int16, c_int32], doc)

doc = """ PICO_STATUS ps2000aSetTriggerChannelProperties
    (
        int16_t                             handle,
        PS2000A_TRIGGER_CHANNEL_PROPERTIES *channelProperties,
        int16_t                             nChannelProperties,
        int16_t                             auxOutputEnable,
        int32_t                             autoTriggerMilliseconds
    ); """
ps2000a.make_symbol("_SetTriggerChannelProperties", "ps2000aSetTriggerChannelProperties", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int16, c_int32], doc)

doc = """ PICO_STATUS ps2000aSetTriggerChannelConditions
    (
        int16_t                     handle,
        PS2000A_TRIGGER_CONDITIONS *conditions,
        int16_t                     nConditions
    ); """
ps2000a.make_symbol("_SetTriggerChannelConditions", "ps2000aSetTriggerChannelConditions", c_uint32,
                    [c_int16, c_void_p, c_int16], doc)

doc = """ PICO_STATUS ps2000aSetTriggerChannelDirections
    (
        int16_t                      handle,
        PS2000A_THRESHOLD_DIRECTION  channelA,
        PS2000A_THRESHOLD_DIRECTION  channelB,
        PS2000A_THRESHOLD_DIRECTION  channelC,
        PS2000A_THRESHOLD_DIRECTION  channelD,
        PS2000A_THRESHOLD_DIRECTION  ext,
        PS2000A_THRESHOLD_DIRECTION  aux
    ); """
ps2000a.make_symbol("_SetTriggerChannelDirections", "ps2000aSetTriggerChannelDirections", c_uint32,
                    [c_int16, c_int32, c_int32, c_int32, c_int32, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps2000aSetTriggerDelay
    (
        int16_t   handle,
        uint32_t  delay
    ); """
ps2000a.make_symbol("_SetTriggerDelay", "ps2000aSetTriggerDelay", c_uint32, [c_int16, c_uint32], doc)

doc = """ PICO_STATUS ps2000aSetPulseWidthQualifier
    (
        int16_t                      handle,
        PS2000A_PWQ_CONDITIONS      *conditions,
        int16_t                      nConditions,
        PS2000A_THRESHOLD_DIRECTION  direction,
        uint32_t                     lower,
        uint32_t                     upper,
        PS2000A_PULSE_WIDTH_TYPE     type
    ); """
ps2000a.make_symbol("_SetPulseWidthQualifier", "ps2000aSetPulseWidthQualifier", c_uint32,
                    [c_int16, c_void_p, c_int16, c_int32, c_uint32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps2000aIsTriggerOrPulseWidthQualifierEnabled
    (
        int16_t  handle,
        int16_t *triggerEnabled,
        int16_t *pulseWidthQualifierEnabled
    ); """
ps2000a.make_symbol("_IsTriggerOrPulseWidthQualifierEnabled", "ps2000aIsTriggerOrPulseWidthQualifierEnabled", c_uint32,
                    [c_int16, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetTriggerTimeOffset64
    (
        int16_t             handle,
        int64_t            *time,
        PS2000A_TIME_UNITS *timeUnits,
        uint32_t            segmentIndex
    ); """
ps2000a.make_symbol("_GetTriggerTimeOffset", "ps2000aGetTriggerTimeOffset64", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS PREF2 PREF3 (ps2000aGetValuesTriggerTimeOffsetBulk64)
    (
        int16_t             handle,
        int64_t            *times,
        PS2000A_TIME_UNITS *timeUnits,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex
    ); """
ps2000a.make_symbol("_GetValuesTriggerTimeOffsetBulk", "ps2000aGetValuesTriggerTimeOffsetBulk64", c_uint32,
                    [c_int16, c_void_p, c_void_p, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS ps2000aGetNoOfCaptures
    (
        int16_t   handle,
        uint32_t *nCaptures
    ); """
ps2000a.make_symbol("_GetNoOfCaptures", "ps2000aGetNoOfCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetNoOfProcessedCaptures
    (
        int16_t   handle,
        uint32_t *nProcessedCaptures
    ); """
ps2000a.make_symbol("_GetNoOfProcessedCaptures", "ps2000aGetNoOfProcessedCaptures", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps2000aSetDataBuffer
    (
        int16_t            handle,
        int32_t            channelOrPort,
        int16_t           *buffer,
        int32_t            bufferLth,
        uint32_t           segmentIndex,
        PS2000A_RATIO_MODE mode
    ); """
ps2000a.make_symbol("_SetDataBuffer", "ps2000aSetDataBuffer", c_uint32,
                    [c_int16, c_int32, c_void_p, c_int32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps2000aSetDataBuffers
    (
        int16_t            handle,
        int32_t            channelOrPort,
        int16_t           *bufferMax,
        int16_t           *bufferMin,
        int32_t            bufferLth,
        uint32_t           segmentIndex,
        PS2000A_RATIO_MODE mode
    ); """
ps2000a.make_symbol("_SetDataBuffers", "ps2000aSetDataBuffers", c_uint32,
                    [c_int16, c_int32, c_void_p, c_void_p, c_int32, c_uint32, c_int32], doc)

doc = """ PICO_STATUS ps2000aSetEtsTimeBuffer
    (
        int16_t  handle,
        int64_t *buffer,
        int32_t  bufferLth
    ); """
ps2000a.make_symbol("_SetEtsTimeBuffer", "ps2000aSetEtsTimeBuffer", c_uint32, [c_int16, c_void_p, c_int32], doc)

doc = """ PICO_STATUS ps2000aIsReady
    (
        int16_t  handle,
        int16_t *ready
    ); """
ps2000a.make_symbol("_IsReady", "ps2000aIsReady", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps2000aRunBlock
    (
        int16_t            handle,
        int32_t            noOfPreTriggerSamples,
        int32_t            noOfPostTriggerSamples,
        uint32_t           timebase,
        int16_t            oversample,
        int32_t           *timeIndisposedMs,
        uint32_t           segmentIndex,
        ps2000aBlockReady  lpReady,
        void              *pParameter
    ); """
ps2000a.make_symbol("_RunBlock", "ps2000aRunBlock", c_uint32,
                    [c_int16, c_int32, c_int32, c_uint32, c_int16, c_void_p, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps2000aRunStreaming
    (
        int16_t             handle,
        uint32_t            *sampleInterval,
        PS2000A_TIME_UNITS  sampleIntervalTimeUnits,
        uint32_t            maxPreTriggerSamples,
        uint32_t            maxPostPreTriggerSamples,
        int16_t             autoStop,
        uint32_t            downSampleRatio,
        PS2000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            overviewBufferSize
    ); """
ps2000a.make_symbol("_RunStreaming", "ps2000aRunStreaming", c_uint32,
                    [c_int16, c_void_p, c_int32, c_uint32, c_uint32, c_int16, c_uint32, c_int32, c_uint32], doc)

doc = """ PICO_STATUS ps2000aGetStreamingLatestValues
    (
        int16_t                handle,
        ps2000aStreamingReady  lpPs2000aReady,
        void                   *pParameter
    ); """
ps2000a.make_symbol("_GetStreamingLatestValues", "ps2000aGetStreamingLatestValues", c_uint32,
                    [c_int16, c_void_p, c_void_p], doc)

doc = """ void *ps2000aStreamingReady
    (
        int16_t   handle,
        int32_t   noOfSamples,
        uint32_t  startIndex,
        int16_t   overflow,
        uint32_t  triggerAt,
        int16_t   triggered,
        int16_t   autoStop,
        void     *pParameter
    );
    define a python function which accepts the correct arguments, and pass it to the constructor of this type.
    """

ps2000a.StreamingReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                         c_int16,
                                                         c_int32,
                                                         c_uint32,
                                                         c_int16,
                                                         c_uint32,
                                                         c_int16,
                                                         c_int16,
                                                         c_void_p)

ps2000a.StreamingReadyType.__doc__ = doc

doc = """ PICO_STATUS ps2000aNoOfStreamingValues
    (
        int16_t   handle,
        uint32_t *noOfValues
    ); """
ps2000a.make_symbol("_NoOfStreamingValues", "ps2000aNoOfStreamingValues", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetMaxDownSampleRatio
    (
        int16_t             handle,
        uint32_t            noOfUnaggreatedSamples,
        uint32_t           *maxDownSampleRatio,
        PS2000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex
    ); """
ps2000a.make_symbol("_GetMaxDownSampleRatio", "ps2000aGetMaxDownSampleRatio", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_int32, c_uint32], doc)

doc = """ PICO_STATUS ps2000aGetValues
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS2000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        int16_t            *overflow
    ); """
ps2000a.make_symbol("_GetValues", "ps2000aGetValues", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetValuesBulk
    (
        int16_t             handle,
        uint32_t           *noOfSamples,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex,
        uint32_t            downSampleRatio,
        PS2000A_RATIO_MODE  downSampleRatioMode,
        int16_t            *overflow
    ); """
ps2000a.make_symbol("_GetValuesBulk", "ps2000aGetValuesBulk", c_uint32,
                    [c_int16, c_void_p, c_uint32, c_uint32, c_uint32, c_int32, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetValuesAsync
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t            noOfSamples,
        uint32_t            downSampleRatio,
        PS2000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        void               *lpDataReady,
        void               *pParameter
    ); """
ps2000a.make_symbol("_GetValuesAsync", "ps2000aGetValuesAsync", c_uint32,
                    [c_int16, c_uint32, c_uint32, c_uint32, c_int32, c_uint32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetValuesOverlapped
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS2000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            segmentIndex,
        int16_t            *overflow
    ); """
ps2000a.make_symbol("_GetValuesOverlapped", "ps2000aGetValuesOverlapped", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetValuesOverlappedBulk
    (
        int16_t             handle,
        uint32_t            startIndex,
        uint32_t           *noOfSamples,
        uint32_t            downSampleRatio,
        PS2000A_RATIO_MODE  downSampleRatioMode,
        uint32_t            fromSegmentIndex,
        uint32_t            toSegmentIndex,
        int16_t            *overflow
    ); """
ps2000a.make_symbol("_GetValuesOverlappedBulk", "ps2000aGetValuesOverlappedBulk", c_uint32,
                    [c_int16, c_uint32, c_void_p, c_uint32, c_int32, c_uint32, c_int32, c_void_p], doc)

doc = """ PICO_STATUS ps2000aStop
    (
        int16_t  handle
    ); """
ps2000a.make_symbol("_Stop", "ps2000aStop", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps2000aHoldOff
    (
        int16_t               handle,
        uint64_t              holdoff,
        PS2000A_HOLDOFF_TYPE  type
    ); """
ps2000a.make_symbol("_HoldOff", "ps2000aHoldOff", c_uint32, [c_int16, c_uint64, c_int32], doc)

doc = """ PICO_STATUS ps2000aGetChannelInformation
    (
        int16_t               handle,
        PS2000A_CHANNEL_INFO  info,
        int32_t               probe,
        int32_t              *ranges,
        int32_t              *length,
        int32_t               channels
    ); """
ps2000a.make_symbol("_GetChannelInformation", "ps2000aGetChannelInformation", c_uint32,
                    [c_int16, c_int32, c_int32, c_int32, c_int32, c_int32], doc)

doc = """ PICO_STATUS ps2000aEnumerateUnits
    (
        int16_t *count,
        int8_t  *serials,
        int16_t *serialLth
    ); """
ps2000a.make_symbol("_EnumerateUnits", "ps2000aEnumerateUnits", c_uint32, [c_void_p, c_char_p, c_void_p], doc)

doc = """ PICO_STATUS ps2000aPingUnit
    (
        int16_t  handle
    ); """
ps2000a.make_symbol("_PingUnit", "ps2000aPingUnit", c_uint32, [c_int16, ], doc)

doc = """ PICO_STATUS ps2000aMaximumValue
    (
        int16_t  handle,
        int16_t *value
    ); """
ps2000a.make_symbol("_MaximumValue", "ps2000aMaximumValue", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps2000aMinimumValue
    (
        int16_t  handle,
        int16_t *value
    ); """
ps2000a.make_symbol("_MinimumValue", "ps2000aMinimumValue", c_uint32, [c_int16, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetAnalogueOffset
    (
        int16_t           handle,
        PS2000A_RANGE     range,
        PS2000A_COUPLING  coupling,
        float            *maximumVoltage,
        float            *minimumVoltage
    ); """
ps2000a.make_symbol("_GetAnalogueOffset", "ps2000aGetAnalogueOffset", c_uint32,
                    [c_int16, c_int32, c_int32, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS ps2000aGetMaxSegments
    (
        int16_t   handle,
        uint32_t *maxSegments
    ); """
ps2000a.make_symbol("_GetMaxSegments", "ps2000aGetMaxSegments", c_uint32, [c_int16, c_void_p], doc)

ps2000a.INI_LOGIC_VOLTS = 1.5

doc = """ void *ps2000aBlockReady
    (
        int16_t    handle,
        PICO_STATUS    status,
        void    *pParameter
    );
    define a python function which accepts the correct arguments, and pass it to the constructor of this type.
    """
    
ps2000a.BlockReadyType = C_CALLBACK_FUNCTION_FACTORY(None,
                                                     c_int16,
                                                     c_uint32,
                                                     c_void_p)

ps2000a.BlockReadyType.__doc__ = doc


#
# Copyright (C) 2020 Pico Technology Ltd. See LICENSE file for terms.
#
"""
This is a Python module defining the functions from the usbPT104Api.h C header
file for Pico USB PT-104 datalogger using the usb PT104 driver API functions.
"""

from ctypes import *
from picosdk.library import Library
from picosdk.ctypes_wrapper import C_CALLBACK_FUNCTION_FACTORY
from picosdk.constants import make_enum

class usbPT104lib(Library):
    def __init__(self):
        super(usbPT104lib, self).__init__("usbPt104")


usbPt104 = usbPT104lib()

usbPt104.PT104_CHANNELS = {
    'USBPT104_CHANNEL_1' : 1,
    'USBPT104_CHANNEL_2' : 2,
    'USBPT104_CHANNEL_3' : 3,
    'USBPT104_CHANNEL_4' : 4,
    'USBPT104_CHANNEL_5' : 5,
    'USBPT104_CHANNEL_6' : 6,
    'USBPT104_CHANNEL_7' : 7,
    'USBPT104_CHANNEL_8' : 8,
    'USBPT104_MAX_CHANNELS' : 8
}

usbPt104.PT104_DATA_TYPE = make_enum([
    'USBPT104_OFF',
    'USBPT104_PT100',
    'USBPT104_PT1000',
    'USBPT104_RESISTANCE_TO_375R',
    'USBPT104_RESISTANCE_TO_10K',
    'USBPT104_DIFFERENTIAL_TO_115MV',
    'USBPT104_DIFFERENTIAL_TO_2500MV',
    'USBPT104_SINGLE_ENDED_TO_115MV',
    'USBPT104_SINGLE_ENDED_TO_2500MV',
    'USBPT104_MAX_DATA_TYPES'
])

usbPt104.IP_DETAILS_TYPE = make_enum([
    'IDT_GET'
    'IDT_SET'
])

def _define_communication_type():
    CT_USB = 0x00000001
    CT_ETHERNET = 0x00000002
    CT_ALL = 0xFFFFFFFF
    
    return {k.upper(): v for k, v in locals().items() if k.startswith("CT")}
    
usbPt104.COMMUNICATION_TYPE = _define_communication_type

doc = """ PICO_STATUS UsbPt104CloseUnit
    (
        int16_t    handle
    ); """
usbPt104.make_symbol("_CloseUnit", "UsbPt104CloseUnit", c_uint32, [c_int16], doc)

doc = """ PICO_STATUS UsbPt104Enumerate
    (
        int8_t    *details,
        uint32_t    *length,
        COMMUNICATION_TYPE    type
    ); """
usbPt104.make_symbol("_Enumerate", "UsbPt104Enumerate", c_uint32, [c_int8, c_uint32, c_uint32], doc)

doc = """ PICO_STATUS UsbPt104GetUnitInfo
    (
        int16_t    handle,
        int8_t    *string,
        int16_t    stringLength,
        int16_t    *requiredSize,
        PICO_INFO    info
    ); """
usbPt104.make_symbol("_GetUnitInfo", "UsbPt104GetUnitInfo", c_uint32, [c_int16, c_void_p, c_int16, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS UsbPt104GetValue
    (
        int16_t    handle,
        USBPT104_CHANNELS    channel,
        int32_t    *value,
        int16_t    filtered
    ); """
usbPt104.make_symbol("_GetValue", "UsbPt104GetValue", c_uint32, [c_int16, c_uint32, c_void_p, c_int16], doc)

doc = """ PICO_STATUS UsbPt104IpDetails
    (
        int16_t    handle,
        int16_t    *enabled,
        int8_t    *ipaddress,
        uint16_t    *length,
        uint16_t    *listeningPort,
        IP_DETAILS_TYPE    type
    ); """
usbPt104.make_symbol("_IpDetails", "UsbPt104IpDetails", c_uint32, [c_int16, c_void_p, c_void_p, c_void_p, c_void_p, c_uint32], doc)

doc = """ PICO_STATUS UsbPt104OpenUnit
    (
        int16_t    *handle,
        int8_t    *serial
    ); """
usbPt104.make_symbol("_OpenUnit", "UsbPt104OpenUnit", c_uint32, [c_void_p, c_void_p], doc)

doc = """ PICO_STATUS UsbPt104OpenUnitViaIp
    (
        int16_t    *handle,
        int8_t    *serial,
        int8_t    *ipAddress
    ); """
usbPt104.make_symbol("_OpenUnitViaIp", "UsbPt104OpenUnitViaIp", c_uint32, [c_void_p, c_void_p, c_void_p], doc)

doc = """ PICO_STATUS UsbPt104SetChannel
    (
        int16_t    handle,
        USBPT104_CHANNELS    channel,
        USBPT104_DATA_TYPES    type,
        int16_t    noOfWires
    ); """
usbPt104.make_symbol("_SetChannel", "UsbPt104SetChannel", c_uint32, [c_int16, c_uint32, c_uint32, c_int16], doc)

doc = """ PICO_STATUS UsbPt104SetMains
    (
        int16_t    handle,
        uint16_t    sixty_hertz
    ); """
usbPt104.make_symbol("_SetMains", "UsbPt104SetMains", c_uint32, [c_int16, c_uint16], doc)



#
# Copyright (C) 2019 Pico Technology Ltd. See LICENSE file for terms.
#
# TC-08 STREAMING MODE EXAMPLE


import ctypes
import numpy as np
import time
from picosdk.usbtc08 import usbtc08 as tc08
from picosdk.functions import assert_pico2000_ok

# Create chandle and status ready for use
chandle = ctypes.c_int16()
status = {}

# open unit
status["open_unit"] = tc08.usb_tc08_open_unit()
assert_pico2000_ok(status["open_unit"])
chandle = status["open_unit"]

# set mains rejection to 50 Hz
status["set_mains"] = tc08.usb_tc08_set_mains(chandle,0)
assert_pico2000_ok(status["set_mains"])

# set up channel
# therocouples types and int8 equivalent
# B=66 , E=69 , J=74 , K=75 , N=78 , R=82 , S=83 , T=84 , ' '=32 , X=88 
typeK = ctypes.c_int8(75)
status["set_channel"] = tc08.usb_tc08_set_channel(chandle, 1, typeK)
assert_pico2000_ok(status["set_channel"])

# get minimum sampling interval in ms
status["get_minimum_interval_ms"] = tc08.usb_tc08_get_minimum_interval_ms(chandle)
assert_pico2000_ok(status["get_minimum_interval_ms"])

# set tc-08 running
status["run"] = tc08.usb_tc08_run(chandle, status["get_minimum_interval_ms"])
assert_pico2000_ok(status["run"])

time.sleep(2)

# collect data 
temp_buffer = (ctypes.c_float * 2 * 15)()
times_ms_buffer = (ctypes.c_int32 * 15)()
overflow = ctypes.c_int16()
status["get_temp"] = tc08.usb_tc08_get_temp(chandle, ctypes.byref(temp_buffer), ctypes.byref(times_ms_buffer), 15, ctypes.byref(overflow), 1, 0, 1)
assert_pico2000_ok(status["get_temp"])

# stop unit
status["stop"] = tc08.usb_tc08_stop(chandle)
assert_pico2000_ok(status["stop"])

# close unit
status["close_unit"] = tc08.usb_tc08_close_unit(chandle)
assert_pico2000_ok(status["close_unit"])

# display status returns
print(status)


#
# Copyright (C) 2018 Pico Technology Ltd. See LICENSE file for terms.
#
from picosdk.errors import DeviceNotFoundError
from picosdk.ps2000 import ps2000
from picosdk.ps2000a import ps2000a
from picosdk.ps3000 import ps3000
from picosdk.ps3000a import ps3000a
from picosdk.ps4000 import ps4000
from picosdk.ps4000a import ps4000a
from picosdk.ps5000a import ps5000a
from picosdk.ps6000 import ps6000
from picosdk.ps6000a import ps6000a


# the A drivers are faster to enumerate devices, so search them first.
drivers = [
    ps2000a,
    ps3000a,
    ps4000a,
    ps5000a,
    ps6000a,
    ps6000,
    ps2000,
    ps3000,
    ps4000,
]


def find_unit():
    """Search for, open and return the first device connected, on any driver."""
    for driver in drivers:
        try:
            device = driver.open_unit()
        except DeviceNotFoundError:
            continue
        return device
    raise DeviceNotFoundError("Could not find any devices on any drivers.")


def find_all_units():
    """Search for, open and return ALL devices on ALL pico drivers (supported in this SDK wrapper)."""
    devices = []
    for driver in drivers:
        try:
            device = driver.open_unit()
        except DeviceNotFoundError:
            continue
        devices.append(device)
    if not devices:
        raise DeviceNotFoundError("Could not find any devices on any drivers.")
    return devices